/* 
 * Autor: Franz Brauße
 * License: GPL v2 
 */

import general.Config;
import general.Log;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.ComponentOrientation;
import java.awt.Dimension;
import java.awt.Frame;
import java.awt.Point;
import java.io.File;
import java.io.IOException;
import java.lang.reflect.Constructor;
import java.lang.reflect.Field;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.lang.IllegalAccessException;
import java.lang.InstantiationException;
import java.net.MalformedURLException;
import java.net.URL;
import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;

import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeCellRenderer;
import javax.swing.tree.TreeNode;
import javax.swing.tree.TreePath;
import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JMenuBar;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTree;
import javax.swing.JToolBar;
import javax.swing.MenuElement;
import javax.swing.UIManager;

import org.jdesktop.jdic.desktop.Desktop;
import org.jdesktop.jdic.desktop.DesktopException;
import org.jdesktop.jdic.tray.SystemTray;
import org.jdesktop.jdic.tray.TrayIcon;

import views.About;
import views.ConnectDialog;
import views.ExceptionBox;
import views.PanelPlugin;
import views.PasswordDialog;
import views.Plugin;
import views.Settings;
import views.ToolbarPlugin;
import views.TrayPlugin;
import yacy.Codings;
import yacy.Peer;
import yacy.event.PeerDisconnectingEvent;
import yacy.exceptions.PConnectionRefusedEx;
import yacy.exceptions.PConnectionResetEx;
import yacy.exceptions.PInterruptionEx;
import yacy.exceptions.PLoginWrongEx;
import yacy.exceptions.PSVNRevisionTooLowEx;
import yacy.exceptions.PServiceUnavailableEx;
import yacy.exceptions.PeerException;

public class YacyAdmin extends JFrame {
	
	private static final long serialVersionUID = 1L;
	
	// IMG_* : 22x22px or larger
	public static final ImageIcon IMG_YACY 			= new ImageIcon("ressources/yacy.png");
	public static final ImageIcon IMG_CONNECT 		= new ImageIcon("ressources/applications-internet.png");
	public static final ImageIcon IMG_DISCONNECT 	= new ImageIcon("ressources/disconnect.png");
	
	// ICON_* : 16x16px
	private static final ImageIcon ICON_SETTINGS 	= new ImageIcon("ressources/configure.png");
	private static final ImageIcon ICON_ABOUT 		= new ImageIcon("ressources/player_playlist.png");
	private static final ImageIcon ICON_ADMIN 		= new ImageIcon("ressources/preferences-system.png");
	private static final ImageIcon ICON_STATUS 		= new ImageIcon("ressources/applications-system.png");
	
	private static final String version 			= "0.42a";
	private static final Config cfg 				= new Config(version);
	
	private static final Config.locales loc 		= new Config.locales(YacyAdmin.class);
	
	private static final String VIEW_THIS 			= "YaCy Admin";
	private static final String VIEW_SETTINGS 		= loc.get("view_settings", 	"Einstellungen");
	private static final String VIEW_ABOUT 			= loc.get("view_about", 		"About");
	
	private static final String URL_YACY_WIKI 		= "http://www.yacy-websuche.de/wiki/index.php/Hauptseite";
	private static final String URL_YACY_ADMIN 		= "http://www.lthawk.de/programmierung.php?view=YacyAdmin";
	
	private static final String HELP 				=
		"run yacy-admin:\n" +
		"  java -cp classes:lib/*.jar YacyAdmin\n" +
		"           [ { -d | -q } ] [ -c ] [ -p ]\n" +
		"           [ -H [ [ PEER_USER: ] PEER_PWD@ ] PEER_HOST [ :PEER_PORT ] ]\n" +
		"\n" +
		"Available command line options:\n" +
		"  -H ...              Specify host to connect on start-up\n" +
		"  PEER_HOST           Hostname of YaCy peer (may be an IP or hostname)\n" +
		"  PEER_USER           Username with admin-rights (not supported by YaCy yet).\n" +
		"                      If not passed, the static admin-account will be used.\n" +
		"  PEER_ADMIN_PWD      Password of the either specified user- or static\n" +
		"                      admin-account of peer. If not given, you will be asked.\n" +
		"  PEER_PORT           The port YaCy runs on. If not given, the standard port\n" +
		"                      8080 will be used.\n" +
		"\n" +
		"  -c,  --no-connect   Don't connect to preset peer on startup. This overrides a\n" +
		"                      possibly specified -H setting\n" +
		"  -d,  --debug        Debugging output and stacktraces will be shown\n" +
		"  -p,  --no-plugins   Disable plugin detection on startup. Only for debugging\n" +
		"  -q,  --quiet        Supress all output, even warnings\n" +
		"  -v,  --no-version-check Don't check peer-version against needed SVN by plugin\n" +
		"  -L,  --license      Prints the license under which this software may be used\n" +
		"  -V,  --version      Prints the program Version and exits immediatly\n" +
		"  -h,  --help         Shows this help\n" +
		"\n" +
		"yacy-admin is alpha-software and may crash or do something definitely not\n" +
		"wanted. Please drop me a line, if it does: <karlchenofhell@mail.berlios.de>.\n";
	
	private static final String LICENSE =
		"All parts of the software known as yacy-admin are covered under the\n" + 
		"terms of the GNU General Public License version 2 as published\n" + 
		"by the Free Software Foundation.\n" + 
		"\n" + 
		"You should have received a copy of the GNU General Public License\n" + 
		"along with this software; if not, write to the Free Software\n" + 
		"Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA";
	
	public static final String ERR_CONNECT 							= loc.get("err_connect", 						"Verbindung fehlgeschlagen");
	public static final String ERR_PASSWORD_WRONG 					= loc.get("err_password_wrong", 				"Das \u00FCbergebene Passwort ist falsch");
	public static final String ERR_CONNECTION_REFUSED 				= loc.get("err_connection_refused", 			"Der Host hat den Verbindungsaufbau abgelehnt.");
	public static final String ERR_CONNECTION_RESET 				= loc.get("err_connection_reset", 				"Der Host hat die Verbindung zurückgesetzt.");
	public static final String ERR_SERVICE_UNAVAILABLE 				= loc.get("err_service_unavailable", 			"YaCy wurde nicht mit SOAP-Support kompiliert. Bitte stellen \nSie dies sicher, indem entweder zus\u00E4tzlich das Ant-Target \n'compileSoap' aufgerufen oder direkt 'ant all' ausgeführt wird.");
	public static final String ERR_CONNECTION_FAILED 				= loc.get("err_connection_failed", 				"Verbindung fehlgeschlagen: %s");
	public static final String ERR_SVN_TOO_LOW 						= loc.get("err_svn_too_low",  					"Die Version des Peers ist zu niedrig (%d < %d); nicht alle ben\u00F6tigten Funktionen werden unterst\u00FCtzt.");
	public static final String ERR_DISCONNECT_FAILED 				= loc.get("err_disconnect_failed", 				"Konnte die Verbindung nicht trennen");
	public static final String ERR_DISCONNECTING 					= loc.get("err_disconnecting", 					"Fehler beim Trennen der Verbindung");
	public static final String ERR_CMD_MISSING_BEHIND_H 			= loc.get("err_cmd_missing_behind_h", 			"Error: You have to specify a hostname after '-H'!");
	public static final String ERR_CMD_UNKNOWN_OPTION 				= loc.get("err_cmd_unknown_option", 			"Unknown command line option '%s'.%n" + "To get an overview of all available options, use '--help'.");
	public static final String ERR_LOAD_ 							= loc.get("err_load_", 							"loading failed: ");
	public static final String ERR_INST_ILLEGAL_CONSTRUCTOR 		= loc.get("err_inst_illegal_constructor", 		"plugin has wrong constructor");
	public static final String ERR_INST_NOT_FOUND 					= loc.get("err_inst_not_found", 				"plugin could not be located on filesystem, check for previous errors");
	public static final String ERR_INST_ABSTRACT 					= loc.get("err_inst_abstract", 					"plugins must not be abstract");
	public static final String ERR_INST_INTERNAL_ERR 				= loc.get("err_inst_internal_err", 				"plugin reported internal error on instantiation");
	public static final String ERR_INST_TRAYPLUGIN 					= loc.get("err_inst_trayplugin", 				"Error on instantiating TrayPlugin \"%s\".");
	public static final String ERR_INST_NO_CONSTRUCTOR 				= loc.get("err_inst_no_constructor", 			"No Constructor found o_O");
	public static final String ERR_INST_NO_SUITABLE_CONSTRUCTOR 	= loc.get("err_inst_no_suitable_constructor", 	"Don't know how to speak with the Plugin (unknown parameters for constructor)");
	public static final String ERR_SETTING_LOOK_AND_FEEL 			= loc.get("err_setting_look_and_feel", 			"setting Look&Feel \"%s\" failed");
	public static final String ERR_BROWSER_NOT_FOUND 				= loc.get("err_browser_not_found", 				"Browser nicht gefunden");
	public static final String ERR_BROWSER_NOT_FOUND_URL 			= loc.get("err_browser_not_found_url", 			"Konnte keinen Browser finden. Die gew\u00FCnschte Seite ist: %s");
	
	public static final String WAR_DISCONNECT_PROBLEM 				= loc.get("war_disconnect_problem", 		ERR_DISCONNECT_FAILED + ": %s%nMöchten Sie die Verbindung trotzdem beenden?");
	public static final String WAR_PLUGIN_SKIPPED_SVN_TOO_LOW 		= loc.get("war_plugin_skipped_svn too_low", "Skipping plugin %s because SVN Revision of connected peer is too low.%nNeeded: %d, found on '%s': %d.");
	
	public static final String INFO_SEARCHING_PLUGINS 				= loc.get("info_searching_plugins", "Searching for plugins");
	public static final String INFO_IS_INTERFACE 					= loc.get("info_is_interface", 		"Interface, skipping...");
	public static final String INFO_CONNECTING_TO 					= loc.get("info_connecting_to", 	"Verbinde zu %s...");
	public static final String INFO_CONNECTED_TO 					= loc.get("info_connected_ to", 	"Erfolgreich verbunden zu %s");
	public static final String INFO_DISCONNECTED_FROM 				= loc.get("info_disconnected_from", "Verbindung zu %s (%s) getrennt");
	public static final String INFO_VALID 							= loc.get("info_valid", 			"(%s) valid");
	public static final String INFO_INVALID 						= loc.get("info_invalid", 			"invalid");
	public static final String INFO_FOUND_CLASS 					= loc.get("info_found_class", 		"Found class: %s: ");
	
	public static final String LBL_USERNAME 						= loc.get("lbl_username", 				"Username");
	public static final String LBL_PASSWORD 						= loc.get("lbl_password", 				"Passwort");
	public static final String LBL_PASSWORD_FOR_HOST 				= loc.get("lbl_password_for_host", 		"Passwort f\u00FCr %s:%d");
	public static final String LBL_CONNECT_TO_YACY_PEER_ 			= loc.get("lbl_connect_to_yacy_peer_", 	"Zu YaCy-Peer verbinden...");
	public static final String LBL_CONNECTION_QUIT 					= loc.get("lbl_connection_quit", 		"Verbindung beenden"); 
	public static final String LBL_FILE 							= loc.get("lbl_file", 					"Datei");
	public static final String LBL_QUIT 							= loc.get("lbl_quit", 					"Beenden");
	public static final String LBL_DISCONNECT 						= loc.get("lbl_disconnect", 			"Verbindung trennen");
	public static final String LBL_CONNECT_ 						= loc.get("lbl_connect", 				"Verbinden...");
	public static final String LBL_LOG_CLEAR 						= loc.get("lbl_log_clear", 				"Log leeren");
	public static final String LBL_HELP 							= loc.get("lbl_help", 					"Hilfe");
	public static final String LBL_HELP_ 							= loc.get("lbl_help_", 					"Hilfe...");
	public static final String LBL_YACY_WIKI 						= loc.get("lbl_yacy_wiki", 				"YaCy Wiki");
	public static final String LBL_YACY_FAQ 						= loc.get("lbl_yacy_faq", 				"YaCy FAQ");
	public static final String LBL_DISPLAY_YACY_ADMIN 				= loc.get("lbl_display_yacy_admin", 	VIEW_THIS + " anzeigen");
	public static final String LBL_SETTINGS_TOOLTIP 				= loc.get("lbl_settings_tooltip", 		VIEW_THIS + "-related settings");
	public static final String LBL_ABOUT_TOOLTIP 					= loc.get("lbl_about_tooltip", 			"About " + VIEW_THIS);
	
	public static final String ACTION_PRESS 						= "PressAction";
	
	private JButton			btnDisconnect	= null;
	private JPanel 			jContentPane 	= null;
	private JPanel 			rPanel 			= null;
	private JPanel			jPanel3			= null;
	private JLabel			jLabel4			= null;
	private JMenu 			menuFile 		= null;
	private JMenu 			menuHelp 		= null;
	private JMenuBar 		menubar 		= null;
	private JMenuItem 		jMIQuit 		= null;
	private JMenuItem 		jMIConnect 		= null;
	private JMenuItem		jMIDisconnect	= null;
	private JMenuItem 		jMIWiki 		= null;
	private JMenuItem 		jMIFaq 			= null;
	private JMenuItem 		jMIHelp 		= null;
	private JMenuItem		tMIQuit			= null;
	private JMenuItem		tMIRestore 		= null;
	private	JPopupMenu		trayMenu		= null;
	private JPopupMenu 		pmLogClean 		= null;
	private JPopupMenu 		pmPeer 			= null;
	private JScrollPane 	jScrollPane 	= null;
	private JSplitPane 		jSplitPane 		= null;
	private JTree 			lTree 			= null;
	private JToolBar		jToolBar		= null;
	private	SystemTray 		sysTray			= null;
	private	TrayIcon		sysTrayIcon		= null;
	
	private DefaultMutableTreeNode 	menu 			= createTree();
	private PeerDMTN 				pm 				= null;
	private PanelPlugin 			activeView 		= null;
	private Peer 					activePeer 		= null;
	private Log 					log 			= null;
	private DefaultListModel 		jLog 			= new DefaultListModel();
	private JScrollPane 			jScrollPane1 	= null;
	private JList 					jList 			= null;
	
	private static final HashMap<String, PanelPluginEntry> 	panelPluginMap 		= new HashMap<String, PanelPluginEntry>();
	private static final ArrayList<ToolbarPlugin> 			toolbarPluginList 	= new ArrayList<ToolbarPlugin>();
	private static final ArrayList<Constructor<TrayPlugin>> trayPluginList 		= new ArrayList<Constructor<TrayPlugin>>();
	
	private JMenuBar getMenubar() {
		if (menubar == null) {
			menubar = new JMenuBar();
			menubar.setComponentOrientation(ComponentOrientation.LEFT_TO_RIGHT);
			menubar.setVisible(true);
			menubar.add(getMenuFile());
			menubar.add(getMenuHelp());
		}
		return menubar;
	}
	
	private JMenu getMenuFile() {
		if (menuFile == null) {
			menuFile = new JMenu();
			menuFile.setText(LBL_FILE);
			menuFile.add(getJMIConnect());
			menuFile.add(getJMIDisconnect());
			menuFile.add(getJMIDelimiter());
			menuFile.add(getJMIQuit());
		}
		return menuFile;
	}
	
	private JMenuItem getJMIQuit() {
		if (jMIQuit == null) {
			jMIQuit = new JMenuItem();
			jMIQuit.setText(LBL_QUIT);
			jMIQuit.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					cfg.save();
					System.exit(0);
				}
			});
		}
		return jMIQuit;
	}
	
	private JMenuItem getJMIDisconnect() {
		if (jMIDisconnect == null) {
			jMIDisconnect = new JMenuItem();
			jMIDisconnect.setText(LBL_DISCONNECT);
			jMIDisconnect.setEnabled(false);
			jMIDisconnect.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					killActiveConnection(activePeer);
				}
			});
		}
		return jMIDisconnect;
	}
	
	private class alConnect implements ActionListener {
		public void actionPerformed(ActionEvent e) {
			ConnectDialog n = new ConnectDialog(YacyAdmin.this, cfg);
			if (n.showDialog())
				YacyAdmin.this.new connectThread(n).start();
		}
	}
	
	private JMenuItem getJMIConnect() {
		if (jMIConnect == null) {
			jMIConnect = new JMenuItem();
			jMIConnect.setText(LBL_CONNECT_);
			jMIConnect.addActionListener(new alConnect());
		}
		return jMIConnect;
	}
	
	private JPopupMenu getPMLogClean() {
		if (pmLogClean == null) {
			pmLogClean = new JPopupMenu();
			JMenuItem d = new JMenuItem(LBL_LOG_CLEAR);
			d.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					log.clear();
				}
			});
			pmLogClean.add(d);
		}
		return pmLogClean;
	}

	private void killActiveConnection(Peer peer) {
		if (peer == null) return;
		String name = peer.getName();
		String host = peer.getHost() + ":" + peer.getPort();
		try {
			peer.firePeerInterruptableEvent(new PeerDisconnectingEvent());
		} catch (PInterruptionEx e) {
			if (JOptionPane.showConfirmDialog(this, String.format(WAR_DISCONNECT_PROBLEM, e.getInterruptionCause())) > 0)
				return;
			if (!e.overrideInterruption())
				JOptionPane.showMessageDialog(
						this,
						ERR_DISCONNECT_FAILED,
						ERR_DISCONNECTING,
						JOptionPane.ERROR_MESSAGE);
		}
		
		for (int i=0; i<menu.getChildCount(); i++)
			if (menu.getChildAt(i) instanceof PeerDMTN && ((PeerDMTN)menu.getChildAt(i)).getYacyPeer() == peer)
				menu.remove(i);
		int peerMenusCount = 0;
		for (int i=0; i<trayMenu.getSubElements().length; i++) {
			if (trayMenu.getSubElements()[i] instanceof PeerMenu) {
				peerMenusCount++;
				if (((PeerMenu)trayMenu.getSubElements()[i]).getYacyPeer() == peer) {
					peerMenusCount--;
					trayMenu.remove(i);
				}
			}
		}
		if (peerMenusCount == 0)
			trayMenu.remove(trayMenu.getComponentCount() - 2);
		
		peer.setUpdateProperties(null, 0);
		lTree = null;
		jScrollPane.setViewportView(getLTree());
		rPanel.removeAll();
		activeView = null;
		activePeer = null;
		jSplitPane.updateUI();
		Iterator<ToolbarPlugin> it = toolbarPluginList.iterator();
		while (it.hasNext())
			((JComponent)it.next()).setEnabled(false);
		jMIDisconnect.setEnabled(false);
		btnDisconnect.setEnabled(false);
		log.setNetworkState(Log.NETWORK_OFFLINE);
		log.addEntry(String.format(INFO_DISCONNECTED_FROM, name, host), null, false);
	}
	
	private JPopupMenu getPMPeer(PeerDMTN clickedPeerMenuItem) {
		if (pmPeer == null) {
			pmPeer = new JPopupMenu();
			JMenuItem d = new JMenuItem(LBL_CONNECTION_QUIT);
			d.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					killActiveConnection(activePeer);
				}
			});
			pmPeer.add(d);
		}
		return pmPeer;
	}
	
	private JMenu getMenuHelp() {
		if (menuHelp == null) {
			menuHelp = new JMenu();
			menuHelp.setText(LBL_HELP);
			menuHelp.add(getJMIWiki());
			// menuHelp.add(getJMIFaq());
			menuHelp.add(getJMIDelimiter());
			menuHelp.add(getJMIHelp());
		}
		return menuHelp;
	}
	
	private void openBrowser(String url) {
		try {
			Desktop.browse(new URL(url));
		} catch (DesktopException e1) {
			if (System.getenv("BROWSER") != null) {
				try {
					Runtime.getRuntime().exec(System.getenv("BROWSER") + " " + url);
				} catch (IOException e2) { cfg.logDebug(e2); }
			} else {
				JOptionPane.showMessageDialog(this, String.format(ERR_BROWSER_NOT_FOUND_URL, url), ERR_BROWSER_NOT_FOUND, JOptionPane.ERROR_MESSAGE);
			}
		} catch (MalformedURLException e1) { cfg.logDebug(e1); }
	}
	
	private JMenuItem getJMIWiki() {
		if (jMIWiki == null) {
			jMIWiki = new JMenuItem();
			jMIWiki.setText(LBL_YACY_WIKI);
			jMIWiki.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					openBrowser(URL_YACY_WIKI);
				}
			});
		}
		return jMIWiki;
	}
	
	private JMenuItem getJMIFaq() {
		if (jMIFaq == null) {
			jMIFaq = new JMenuItem();
			jMIFaq.setText(LBL_YACY_FAQ);
		}
		return jMIFaq;
	}
	
	private JMenuItem getJMIHelp() {
		if (jMIHelp == null) {
			jMIHelp = new JMenuItem();
			jMIHelp.setText(LBL_HELP_);
			jMIHelp.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					openBrowser(URL_YACY_ADMIN);
				}
			});
		}
		return jMIHelp;
	}

	private JMenuItem getJMIDelimiter() {
		JMenuItem jMIDelimiter = new JMenuItem();
		jMIDelimiter.setEnabled(false);
		return jMIDelimiter;
	}
	
	private JPanel getJContentPane() {
		if (jContentPane == null) {
			jContentPane = new JPanel();
			jContentPane.setLayout(new BorderLayout());
			jContentPane.add(getJSplitPane(), BorderLayout.CENTER);
			jContentPane.add(getJScrollPane1(), BorderLayout.SOUTH);
			jContentPane.add(getJPanel3(), BorderLayout.NORTH);
		}
		return jContentPane;
	}

	private JSplitPane getJSplitPane() {
		if (jSplitPane == null) {
			jSplitPane = new JSplitPane();
			jSplitPane.setRightComponent(getRPanel());
			jSplitPane.setLeftComponent(getJScrollPane());
		}
		return jSplitPane;
	}
	
	protected JPanel getRPanel() {
		if (rPanel == null) {
			rPanel = new JPanel();
			rPanel.setLayout(new BorderLayout());
		}
		return rPanel;
	}
	
	private DefaultMutableTreeNode createTree() {
		DefaultMutableTreeNode top = new DefaultMutableTreeNode(VIEW_THIS);
		
		DefaultMutableTreeNode yadmin = new DefaultMutableTreeNode(VIEW_THIS);
		top.add(yadmin);
		
		DefaultMutableTreeNode yaSettings = new DefaultMutableTreeNode(VIEW_SETTINGS);
		DefaultMutableTreeNode yaAbout = new DefaultMutableTreeNode(VIEW_ABOUT);
		yadmin.add(yaSettings);
		yadmin.add(yaAbout);
		
		return top;
	}
	
	private class PeerDMTN extends DefaultMutableTreeNode {
		
		static final long serialVersionUID = 1L;
		
		private Peer thePeer = null;
		
		public PeerDMTN(Peer peer) 		{ this.thePeer = peer; }
		public String 	toString() 		{ return this.thePeer.getName(); }
		public Peer 	getYacyPeer() 	{ return this.thePeer; }
	}
	
	private PeerDMTN createPluginTree(Peer peer) {
		PeerDMTN top = new PeerDMTN(peer);
		HashMap<String,DefaultMutableTreeNode> catPluginMap = new HashMap<String,DefaultMutableTreeNode>();
		for (int i=0; i<PanelPlugin.menuCategories.length; i++)
			catPluginMap.put(PanelPlugin.menuCategories[i], new DefaultMutableTreeNode(PanelPlugin.menuCategories[i]));
		
		Iterator<String> it = panelPluginMap.keySet().iterator();
		while (it.hasNext()) {
			String e = (String)it.next();						// name of plugin
			PanelPluginEntry ea = panelPluginMap.get(e);
			if (!cfg.noVersionCheck && ea.getNeededRev() > peer.getSVNVersion()) {		// if desired SVN doesn't fit
				cfg.logDebug(String.format(WAR_PLUGIN_SKIPPED_SVN_TOO_LOW,
						e, ea.getNeededRev(), peer.getName(), peer.getSVNVersion()));
				continue;										// dont't add plugin
			}
			int i;
			for (i=0; i < PanelPlugin.menuCategories.length; i++)
				if (ea.getMenupath().equals(PanelPlugin.menuCategories[i]))
					break;
			catPluginMap.get((i < PanelPlugin.menuCategories.length) ? PanelPlugin.menuCategories[i] : PanelPlugin.OTHER)
					.add(new DefaultMutableTreeNode(e));
		}
		
		for (String d : catPluginMap.keySet())
			if (catPluginMap.get(d).getChildCount() > 0)
				top.add(catPluginMap.get(d));
		
		return top;
	}
	
	private void addPeer(Peer peer) {
		pm = createPluginTree(peer);
		menu.add(pm);
		lTree = null;
		jScrollPane.setViewportView(getLTree());
		int peerMenusCount = 0;
		for (MenuElement d : trayMenu.getSubElements())
			if (d instanceof PeerMenu)
				peerMenusCount++;
		if (peerMenusCount == 0) {
			JMenuItem d = new JMenuItem();
			d.setEnabled(false);
			trayMenu.add(d, trayMenu.getComponentCount() - 1);
		}
		trayMenu.add(getPeerTrayMenu(peer), trayMenu.getComponentCount() - 2);
		lTree.setSelectionPath(new TreePath(pm.getPath()));
	}
	
	private class connectThread extends Thread {
		
		private final String 		host;
		private final int 			port;
		private final String 		pwd;
		private final String 		user;
		private final ConnectDialog cd;
		private boolean result 	= false;
		
		public connectThread(ConnectDialog cd) {
			this.host = cd.getHost();
			this.port = cd.getPort();
			this.user = cd.getUser();
			this.pwd = cd.getPwd();
			this.cd = cd;
		}
		
		public connectThread(String host, int port, String user, String pwd) {
			this.host = host;
			this.port = port;
			this.user = user;
			this.pwd = pwd;
			this.cd = null;
		}
		
		public boolean getSuccess() { return this.result; }
		
		public void run() {
			String host = this.host + ':' + this.port;
			YacyAdmin.this.log.addEntry(String.format(INFO_CONNECTING_TO, host), host, true);
			YacyAdmin.this.log.setNetworkBusy();
			
			try {
				Peer peer = new Peer(this.host, this.port, this.user, this.pwd, cfg);
				YacyAdmin.this.log.setNetworkIdle();
				YacyAdmin.this.addPeer(peer);
				YacyAdmin.this.log.addEntry(String.format(INFO_CONNECTED_TO, peer.getName()), host, false);
				this.result = true;
			} catch (PeerException e) {
				YacyAdmin.this.log.setNetworkError();
				String err = (
						(e instanceof PLoginWrongEx) 			? YacyAdmin.ERR_PASSWORD_WRONG :
						(e instanceof PSVNRevisionTooLowEx) 	? YacyAdmin.ERR_SVN_TOO_LOW :
						(e instanceof PConnectionRefusedEx) 	? YacyAdmin.ERR_CONNECTION_REFUSED :
						(e instanceof PServiceUnavailableEx) 	? YacyAdmin.ERR_SERVICE_UNAVAILABLE :
						(e instanceof PConnectionResetEx) 		? YacyAdmin.ERR_CONNECTION_RESET :
						null);
				YacyAdmin.this.log.addEntry((err == null) ? e.getMessage() : err, host, false);
				if (err == null)									// an unknown PeerException? That must not be...
					new ExceptionBox(YacyAdmin.this, e);
				else												// a yet known Exception with explanation
					JOptionPane.showMessageDialog(YacyAdmin.this, err, YacyAdmin.ERR_CONNECT, JOptionPane.ERROR_MESSAGE);
			} catch (RemoteException e) {
				new ExceptionBox(YacyAdmin.this, e);				// another unknown exception, we need the stacktrace!
			}
			
			// only save host/password if successfully connected and we if have the ability to
			if (this.result && this.cd != null) cd.savePassword();
		}
	}
	
	private JScrollPane getJScrollPane() {
		if (jScrollPane == null) {
			jScrollPane = new JScrollPane();
			jScrollPane.setViewportView(getLTree());
		}
		return jScrollPane;
	}
	
	private void openTreeItems(TreeNode root) {
		for (int i=0; i<root.getChildCount(); i++) {
			if (root.getChildAt(i).isLeaf()) continue;
			openTreeItems(root.getChildAt(i));
		}
		lTree.expandPath(new TreePath(((DefaultMutableTreeNode)root).getPath()));
	}
	
	
	private static final class lTreeCellRenderer extends DefaultTreeCellRenderer {
		
		private static final long serialVersionUID = 1L;

		public Component getTreeCellRendererComponent(
				JTree tree,
				Object value,
                boolean sel,
                boolean expanded,
                boolean leaf,
                int row,
                boolean hasFocus) {
			
			super.getTreeCellRendererComponent(
					tree, value, sel,
					expanded, leaf, row,
					hasFocus);
			
			if (leaf) {
				this.setIcon(null);
				PanelPluginEntry oc = panelPluginMap.get(value.toString());
				if (oc != null) {
					try {
						this.setIcon(oc.getIcon());
						this.setToolTipText(oc.getTooltip());
					} catch (Exception e) {
						YacyAdmin.cfg.log(e);
					}
				} else if (value.toString().equals(VIEW_SETTINGS)) {
					this.setIcon(ICON_SETTINGS);
					this.setToolTipText(LBL_SETTINGS_TOOLTIP);
				} else if (value.toString().equals(VIEW_ABOUT)) {
					this.setIcon(ICON_ABOUT);
					this.setToolTipText(LBL_ABOUT_TOOLTIP);
				}
			} else if (value instanceof PeerDMTN) {
				// the peer-node itself
			} else {
				if (value.toString().equals(PanelPlugin.ADMIN)) {
					this.setIcon(ICON_ADMIN);
				} else if (value.toString().equals(PanelPlugin.STATUS)) {
					this.setIcon(ICON_STATUS);
				}
			}
			return this;
		}
	}
	
	private JTree getLTree() {
		if (lTree == null) {
			lTree = new JTree(menu);
			openTreeItems(menu);
			lTree.setRootVisible(false);
			lTree.setPreferredSize(new Dimension(150, 0));
			lTree.setShowsRootHandles(true);
			
			// used for symbols next to entries and further appearance
			lTree.setCellRenderer(new lTreeCellRenderer());
			
			// provides the context menu for PeerDMTN objects
			lTree.addMouseListener(new java.awt.event.MouseAdapter() {
				public void mouseClicked(java.awt.event.MouseEvent e) {
					if (lTree.getSelectionCount() == 0) return;
					if (e.getButton() == java.awt.event.MouseEvent.BUTTON3)	{	// rechte Maustaste
						Object where = (DefaultMutableTreeNode)lTree.getSelectionPath().getPathComponent(1);
						if (lTree.getSelectionPath().getLastPathComponent() == where && where instanceof PeerDMTN) {
							getPMPeer((PeerDMTN)where).show(lTree, e.getX(), e.getY());
						}
						return;
					}
				}
			});
			
			// cares about (un-)loading plugins and visibility of items based on selection
			lTree.addTreeSelectionListener(new TreeSelectionListener() {
				public void valueChanged(TreeSelectionEvent e) {
					if (lTree.getSelectionCount() == 0) return;
					
					Object where = (DefaultMutableTreeNode)lTree.getSelectionPath().getPathComponent(1);
					String what = lTree.getSelectionPath().getLastPathComponent().toString();
					
					Peer tmpPeer = activePeer;
					if (where instanceof PeerDMTN) {
						activePeer = ((PeerDMTN)where).getYacyPeer();
						jMIDisconnect.setEnabled(true);
						btnDisconnect.setEnabled(true);
					} else {
						activePeer = null;
						jMIDisconnect.setEnabled(false);
						btnDisconnect.setEnabled(false);
					}
					
					if (tmpPeer != activePeer) {				// active peer has changed
						Iterator<ToolbarPlugin> it = toolbarPluginList.iterator();
						while (it.hasNext()) {
							ToolbarPlugin d = it.next();
							d.init(activePeer);
							if (activePeer == null)
								((JComponent)d).setEnabled(false);
						}
						jToolBar.updateUI();
						if (tmpPeer != null)
							tmpPeer.setUpdateProperties(null, 0);
					}
					
					if (lTree.getSelectionPath().getLastPathComponent() != null) {			// XXX: is this if needed?
						
						rPanel.removeAll();
						if (((DefaultMutableTreeNode)lTree.getSelectionPath().getLastPathComponent()).isLeaf()) {
							
							try {
								activeView = null;
								if (where instanceof PeerDMTN) {			// Peer Tree
									PanelPluginEntry oc = panelPluginMap.get(what);
									if (oc == null) { return; }				// no plugin loaded and Peer-root is leaf
									PanelPlugin plc = oc.getConstructor().newInstance(new Object[0]);
									
									activeView = plc;
									rPanel.add((JComponent)plc);
									rPanel.repaint();
									activeView.init(activePeer, cfg);
									if (activePeer != null) {
										if (cfg.useGlobalUpdateDelay && activeView.getUpdateDelay() > 0)
											activePeer.setUpdateProperties(activeView, cfg.globalUpdateDelay);
										else
											activePeer.setUpdateProperties(activeView, activeView.getUpdateDelay());
									}
								} else {									// Program Tree
									if (what.equals(VIEW_SETTINGS)) {
										rPanel.add(new Settings(cfg));
									} else if (what.equals(VIEW_ABOUT)) {
										rPanel.add(new About());
									}
								}
							} catch (Exception f) { cfg.log(f); }
						}
					}
					jSplitPane.updateUI();
				}
			});
		}
		return lTree;
	}
	
	private JScrollPane getJScrollPane1() {
		if (jScrollPane1 == null) {
			jScrollPane1 = new JScrollPane();
			jScrollPane1.setPreferredSize(new Dimension(0, 60));
			jScrollPane1.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
			jScrollPane1.setVisible(false);
			jScrollPane1.setViewportView(getJList());
		}
		return jScrollPane1;
	}

	private JList getJList() {
		if (jList == null) {
			jList = new JList(jLog);
			jList.setVisible(true);
			jList.addComponentListener(new java.awt.event.ComponentAdapter() {
				public void componentResized(java.awt.event.ComponentEvent e) {
					jScrollPane1.getViewport().setViewPosition(new Point(0,jList.getHeight()));
				}
			});
			jList.addMouseListener(new java.awt.event.MouseAdapter() {
				public void mouseClicked(java.awt.event.MouseEvent e) {
					if (e.getButton() == java.awt.event.MouseEvent.BUTTON3)	{	// rechte Maustaste
						getPMLogClean().show(jList, e.getX(), e.getY());
					}
				}
			});
		}
		return jList;
	}
	
	private JPanel getJPanel3() {
		if (jPanel3 == null) {
			jLabel4 = new JLabel("");
			jLabel4.setMinimumSize(new Dimension(24, 24));
			jLabel4.setMaximumSize(new Dimension(24, 24));
			jLabel4.setPreferredSize(new Dimension(24, 24));
			jLabel4.setIconTextGap(0);
			BorderLayout borderLayout1 = new BorderLayout();
			borderLayout1.setHgap(0);
			jPanel3 = new JPanel();
			jPanel3.setPreferredSize(new Dimension(42, 24));
			jPanel3.setLayout(borderLayout1);
			jPanel3.add(getJToolBar2(), BorderLayout.CENTER);
			jPanel3.add(jLabel4, BorderLayout.EAST);
		}
		return jPanel3;
	}

	private JToolBar getJToolBar2() {
		if (jToolBar == null) {
			jToolBar = new JToolBar();
			jToolBar.setMaximumSize(new Dimension(30, 28));
			jToolBar.setPreferredSize(new Dimension(18, 24));
			jToolBar.setMinimumSize(new Dimension(30, 28));
			jToolBar.setFloatable(false);
			
			JButton cnt = new JButton("");
			cnt.setMinimumSize(new Dimension(24, 24));
			cnt.setMaximumSize(new Dimension(24, 24));
			cnt.setPreferredSize(new Dimension(24, 24));
			cnt.setToolTipText(LBL_CONNECT_TO_YACY_PEER_);
			cnt.setBorderPainted(false);
			cnt.setIcon(IMG_CONNECT);
			cnt.addActionListener(new alConnect());
			jToolBar.add(cnt);
			jToolBar.add(getBtnDisconnect());
		}
		return jToolBar;
	}
	
	private JButton getBtnDisconnect() {
		if (btnDisconnect == null) {
			btnDisconnect = new JButton();
			btnDisconnect.setEnabled(false);
			btnDisconnect.setMinimumSize(new Dimension(24, 24));
			btnDisconnect.setPreferredSize(new Dimension(24, 24));
			btnDisconnect.setToolTipText(LBL_DISCONNECT);
			btnDisconnect.setBorderPainted(false);
			btnDisconnect.setIcon(IMG_DISCONNECT);
			btnDisconnect.setMaximumSize(new Dimension(24, 24));
			btnDisconnect.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					if (activePeer != null)
						killActiveConnection(activePeer);
				}
			});
		}
		return btnDisconnect;
	}
	
	private JPopupMenu getTrayMenu() {
		if(trayMenu == null) {
			trayMenu = new JPopupMenu();
			trayMenu.add(getTMIRestore());
			trayMenu.add(getTMIQuit());
		}
		return trayMenu;
	}
	
	private JMenuItem getTMIQuit() {
		if(tMIQuit == null) {
			tMIQuit = new JMenuItem();
			tMIQuit.setText(LBL_QUIT);
			tMIQuit.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					cfg.save();
					System.exit(0);
				}
			});
		}
		return tMIQuit;
	}
	
	private JMenuItem getTMIRestore() {
		if(tMIRestore == null) {
			tMIRestore = new JMenuItem();
			tMIRestore.setText(LBL_DISPLAY_YACY_ADMIN);
			tMIRestore.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					restoreFromTray();
				}
			});
		}
		return tMIRestore;
	}
	
	private TrayIcon getSysTrayIcon() {
		if(sysTrayIcon == null) {
			sysTrayIcon = new TrayIcon(IMG_YACY, VIEW_THIS, getTrayMenu());
			sysTrayIcon.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					if (e.getActionCommand().equals(ACTION_PRESS)) {
						restoreFromTray();
					}
				}
			});
		}
		return sysTrayIcon;
	}
	
	private void setupTrayIcon() {
		if(sysTray == null)
			sysTray = SystemTray.getDefaultSystemTray();
		sysTray.addTrayIcon(getSysTrayIcon());
	}
	
	private void restoreFromTray() {
		this.setVisible(true);
		this.setState(Frame.NORMAL);
		this.toFront();
		this.removeTrayIcon();
	}
	
	private void removeTrayIcon() {
		if (sysTray == null || sysTrayIcon == null) return;
		sysTray.removeTrayIcon(sysTrayIcon);
	}

	public static void main(String[] args) {
		cfg.load();
		
		if (new File("client-config.wsdd").exists())
			System.setProperty("axis.ClientConfigFile", "client-config.wsdd");
		
		// parse command-line, overwrites default settings in cfg
		for (int i=0; i<args.length; i++) {
			String d = args[i];
			
			if (d.equals("--no-connect") || d.equals("-c")) {			// don't auto-connect
				cfg.noConnectOnStartup = true;
				continue;
				
			} else if (d.equals("--debug") || d.equals("-d")) {			// Debug Mode
				cfg.logLevel = Config.LOG_DEBUG;
				continue;
				
			} else if (d.equals("--no-plugins") || d.equals("-p")) {	// no plugins
				cfg.loadPlugins = false;
				continue;
				
			} else if (d.equals("--quiet") || d.equals("-q")) {			// Quiet Mode
				cfg.logLevel = Config.LOG_QUIET;
				continue;
				
			} else if (d.equals("--no-version-check") || d.equals("-v")) {
				cfg.noVersionCheck = true;
				continue;
				
			} else if (d.equals("-H")) {								// connect to host
				if (i == args.length - 1) {
					System.err.println(ERR_CMD_MISSING_BEHIND_H);
					System.exit(1);
				}
				if (args[i + 1].indexOf('@') > -1) {
					String userpart = args[i + 1].substring(0, args[i +1].indexOf('@'));
					if (userpart.indexOf(':') > -1) {
						cfg.connectOnStartupOverrideUser = userpart.substring(0, userpart.indexOf(':'));
						cfg.connectOnStartupOverridePwd = userpart.substring(userpart.indexOf(':') + 1);
					} else
						cfg.connectOnStartupOverrideUser = userpart;
					cfg.connectOnStartupOverride = args[i + 1].substring(args[i + 1].indexOf('@') + 1);
				} else {
					cfg.connectOnStartupOverride = args[i + 1];
				}
				if (cfg.connectOnStartupOverride.indexOf(':') == -1)
					cfg.connectOnStartupOverride += ":8080";
				i++;
				continue;
				
			} else if (d.equals("--license") || d.equals("-L")) {
				// License
				System.out.println(LICENSE);
				System.exit(0);
				
			} else if (d.equals("--version") || d.equals("-V")) {
				System.out.println(YacyAdmin.version);
				System.exit(0);
				
			} else if (d.equals("--help") || d.equals("-h")) {
				// Help
				System.out.println(HELP);
				System.exit(0);
				
			} else {
				// command line error
				System.out.println(String.format(ERR_CMD_UNKNOWN_OPTION, d));
				System.exit(1);
			}
		}
		
		if (cfg.loadPlugins)
			loadPlugins("classes" + File.separator + "plugins");
		
		new YacyAdmin();
	}
	
	private static final class PanelPluginEntry {
		private final Constructor<PanelPlugin> constructor;
		private final String menupath;
		private final int neededRev;
		private final ImageIcon icon;
		private final String tooltip;
		
		public PanelPluginEntry(
				Constructor<PanelPlugin> constructor,
				String menupath,
				int neededRev,
				ImageIcon icon,
				String tooltip) {
			this.constructor = constructor;
			this.menupath = menupath;
			this.neededRev = neededRev;
			this.icon = icon;
			this.tooltip = tooltip;
		}
		
		public Constructor<PanelPlugin> getConstructor() { return this.constructor; }
		public String getMenupath() { return this.menupath; }
		public int getNeededRev() { return this.neededRev; }
		public ImageIcon getIcon() { return this.icon; }
		public String getTooltip() { return this.tooltip; }
	}
	
	private static int loadPlugins(final String path) {
		int count = 0;
		/* find all Plugins */
		cfg.logDebugAdditional(INFO_SEARCHING_PLUGINS, " in " + path + File.separator, true);
		File t = new File(path);
		
		if (t.exists()) {
			final String[] x = t.list();
			
			Constructor[] ccs;
			Object co;
			File b;
			Class c;
			int ccuse;
			
			for (int i=0; i<x.length; i++) {
				b = new File(path + File.separator + x[i]);
				
				if (b.isFile() && b.canRead() && b.getName().endsWith(".class") && b.getName().indexOf("$") == -1) {
					cfg.log(String.format(INFO_FOUND_CLASS, b.getName().substring(0, b.getName().indexOf(".class"))));
					
					try {
						// load class
						c = Class.forName("plugins." + b.getName().substring(0, b.getName().indexOf(".class")));
						
						if (c.isInterface()) {
							cfg.logDebug(INFO_IS_INTERFACE, true);
							continue;
						}
						
						// select the constructor taking no arguments
						ccs = c.getConstructors();
						for (ccuse=0; ccuse<ccs.length; ccuse++)
							if (ccs[ccuse].getParameterTypes().length == 0) break; 
						
						if (ccs.length == 0) {
							cfg.logDebug(ERR_INST_NO_CONSTRUCTOR, true);
							continue;
						} else if (ccuse == ccs.length) {
							cfg.logDebug(ERR_INST_NO_SUITABLE_CONSTRUCTOR);
							continue;
						}
						
						// create new object to test whether it is usable as Plugin
						co = ccs[ccuse].newInstance(new Object[0]);
						
						cfg.logDebug("\n" +
								"Created new instance: " + co.toString() + "\n" +
								"It appears to be: ");
						
						if (co instanceof Plugin && co instanceof JComponent) {
							if (co instanceof PanelPlugin) {
								cfg.log(String.format(INFO_VALID, PanelPlugin.class.getName()));
								// store constructor under it's description-name needed for the menu
								panelPluginMap.put(
										((PanelPlugin)co).getDescription(),
										new PanelPluginEntry(ccs[ccuse],
												((PanelPlugin)co).getMenuPath(),
												Integer.valueOf(((PanelPlugin)co).getNeededYacyRevision()),
												((PanelPlugin)co).getMenuIcon(),
												((PanelPlugin)co).getMenuToolTip()));
							}
							if (co instanceof ToolbarPlugin) {
								cfg.log(String.format(INFO_VALID, ToolbarPlugin.class.getName()));
								// store object itself, because it needs to be displayed and therefore
								// instantiated only once
								toolbarPluginList.add((ToolbarPlugin)co);
							}
							if (co instanceof TrayPlugin) {
								cfg.log(String.format(INFO_VALID, TrayPlugin.class.getName()));
								// store constructor, because the plugin must be instantiated newly for
								// every new connection
								trayPluginList.add(ccs[ccuse]);
							}
							count++;
						} else {
							cfg.logDebugAdditional("invalid", " (" +
									((co instanceof PanelPlugin) ? ("") : ("no ")) + "instance of PanelPlugin / " +
									((co instanceof ToolbarPlugin) ? ("") : ("no ")) + "instance of ToolbarPlugin / " +
									((co instanceof TrayPlugin) ? ("") : ("no ")) + "instance of TrayPlugin / " +
									((co instanceof JComponent) ? ("") : ("no ")) + "instance of JComponent)");
						}
					}
					catch (SecurityException e) 		{ cfg.logConsole(ERR_LOAD_, e); }
					catch (IllegalArgumentException e) 	{ cfg.logConsole(ERR_LOAD_ + ERR_INST_ILLEGAL_CONSTRUCTOR, e); }
					catch (ClassNotFoundException e) 	{ cfg.logConsole(ERR_LOAD_ + ERR_INST_NOT_FOUND, e); }
					catch (InstantiationException e) 	{ cfg.logConsole(ERR_LOAD_ + ERR_INST_ABSTRACT, e); }
					catch (IllegalAccessException e) 	{ cfg.logConsole(ERR_LOAD_, e); }
					catch (InvocationTargetException e) { cfg.logConsole(ERR_LOAD_, e.getCause()); }
					
					cfg.log(null, true);
				}												// end if {valid file}
			}													// end for()
		}														// end if t.exists()
		return count;
	}
	
	public YacyAdmin() {
		super();
		cfg.setParent(this);
		initialize();
		this.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		this.setLocationByPlatform(true);
		this.setVisible(true);
	}
	
	private void initialize() {
		try {
			UIManager.setLookAndFeel(UIManager.getCrossPlatformLookAndFeelClassName());
		} catch (Exception e) {
			cfg.logConsole(String.format(ERR_SETTING_LOOK_AND_FEEL, UIManager.getCrossPlatformLookAndFeelClassName()), e);
		}
		this.setSize(cfg.WindowWidth,cfg.WindowHeight);
		this.setJMenuBar(getMenubar());
		this.setContentPane(getJContentPane());
		this.setIconImage(IMG_YACY.getImage());
		
		this.setTitle(VIEW_THIS);
		this.addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				cfg.WindowWidth = getWidth();
				cfg.WindowHeight = getHeight();
				cfg.save();
			}
			public void windowIconified(WindowEvent e) {
				setVisible(false);
				setupTrayIcon();
			}
		});
		
		log = new Log(cfg, jContentPane, jScrollPane1, jLog, jLabel4);
		log.setNetworkOffline();
		cfg.log = log;
		
		for (ToolbarPlugin d : toolbarPluginList) {
			d.init(cfg);
			jToolBar.add((JComponent)d);
		}
		getTrayMenu();
		
		if (!cfg.noConnectOnStartup) {
			if (cfg.connectOnStartupOverride != null && cfg.connectOnStartupOverride.length() > 0) {
				String host = cfg.connectOnStartupOverride.substring(0, cfg.connectOnStartupOverride.indexOf(':'));
				int port = Integer.parseInt(cfg.connectOnStartupOverride.substring(cfg.connectOnStartupOverride.indexOf(':') + 1));
				String pwd = null;
				String user = cfg.connectOnStartupOverrideUser;
				
				if (cfg.connectOnStartupOverridePwd == null) {
					PasswordDialog pd = new PasswordDialog(String.format(LBL_PASSWORD_FOR_HOST, host, port));
					pd.setNameLabel(LBL_USERNAME);
					pd.setName((user == null) ? Peer.DEFAULT_USER : user);
					pd.setPassLabel(LBL_PASSWORD);
					pd.setModal(true);
					if (pd.showDialog()) {
						cfg.connectOnStartupOverridePwd = pd.getPass();
						user = pd.getName();
					} else return;
				}
				
				if (user == null || user.length() == 0) user = Peer.DEFAULT_USER;
				pwd = Codings.encodeMD5Hex(Codings.encodeString(user + ':' + cfg.connectOnStartupOverridePwd));
				cfg.log(String.format(INFO_CONNECTING_TO, cfg.connectOnStartupOverride), true);
				new connectThread(host, port, user, pwd).start();
			}
			
			if (cfg.connectOnStartup.length() > 0) {
				cfg.log(String.format(INFO_CONNECTING_TO, cfg.connectOnStartup), true);
				String host = cfg.connectOnStartup.substring(0, cfg.connectOnStartup.indexOf(':'));
				String user = Peer.DEFAULT_USER;
				if (host.indexOf('@') > -1) {
					user = host.substring(0, host.indexOf('@'));
					host = host.substring(host.indexOf('@') + 1);
				}
				int port = Integer.parseInt(cfg.connectOnStartup.substring(cfg.connectOnStartup.indexOf(':') + 1));
				String pwd = cfg.connects.get(cfg.connectOnStartup);
				new connectThread(host, port, user, pwd).start();
			}
		}
	}
	
	private class PeerMenu extends JMenu {
		
		private static final long serialVersionUID = 1L;
		
		private Peer peer = null;
		
		public PeerMenu(Peer yacyPeer) {
			super(yacyPeer.getName());
			this.peer = yacyPeer;
		}
		public String toString() { return this.peer.getName(); }
		public Peer getYacyPeer() { return this.peer; }
	}
	
	private PeerMenu getPeerTrayMenu(Peer peer) {
		PeerMenu r = new PeerMenu(peer);
		
		for (Constructor<TrayPlugin> d : trayPluginList) {
			try {
				TrayPlugin f = d.newInstance(new Object[0]);
				f.init(peer, cfg);
				if (f instanceof JMenu)
					r.add((JMenu)f);
				else if (f instanceof JMenuItem)
					r.add((JMenuItem)f);
				else
					r.add((JComponent)f);
			} catch (Exception e) {
				cfg.log(String.format(ERR_INST_TRAYPLUGIN, d.getName()), e);
			}
		}
		return r;
	}
}
