package plugins;

import general.Config;

import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.net.MalformedURLException;
import java.net.URL;
import java.rmi.RemoteException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;

import javax.swing.DefaultListModel;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.JTextField;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import org.jdesktop.jdic.desktop.Desktop;
import org.jdesktop.jdic.desktop.DesktopException;

import views.PanelPlugin;
import yacy.Peer;
import yacy.datatypes.Bookmark;


public class Bookmarks extends JPanel implements views.PanelPlugin {

	private static final long serialVersionUID = 1L;
	private static final Insets DEF_INSETS = new Insets(5, 5, 5, 5);
	
	public static final ImageIcon ICON_BOOKMARKS = new ImageIcon("ressources/bookmarksicon.png");
	
	private	JPanel					EditBMPanel;
	private	JTextField				EditBMURLTextField;
	private	JTextField				EditBMTitleTextField;
	private	JTextField				EditBMDescrTextField;
	private	JTextField				EditBMTagsTextField;
	private	JButton					EditBMAddButton;
	private	JButton					EditBMEditButton;
	
	private JDialog					DeleteConfirmDialog;
	private	JPanel					DeleteConfirmInfoPanel;
	private	JLabel					DeleteConfirmInfoBMTitle;
	private	JLabel					DeleteConfirmInfoBMURL;
	private	JPanel					DeleteConfirmControlPanel;
	private	JButton					DeleteConfirmControlDeleteButton;
	private	JButton					DeleteConfirmControlDenyButton;
	
	private	JDialog					NewBMDialog;
	private	JPanel					NewBMEditPanel;
	private	JTextField				NewBMURLTextField;
	private	JTextField				NewBMTitleTextField;
	private	JTextField				NewBMDescrTextField;
	private	JTextField				NewBMTagsTextField;
	private JCheckBox				NewBMPublicCheckBox;
	private	boolean					NewBMPublicCheckBoxState;
	private	JPanel					NewBMControlPanel;
	private	JButton					NewBMSaveButton;
	private	JButton					NewBMCancelButton;
	
	private	JTabbedPane				ShowBMTabbedPane;
	private	JPanel					ShowByBMPanel;
	private	JPanel					ShowByBMControlPanel;
	private	JButton					ShowByBMRefreshButton;
	private	JButton					ShowByBMDeleteButton;
	private	JList					ShowByBMList;
	private	DefaultListModel		ShowByBMListContent;
	
	private Peer					peer;
	private Config					cfg;
	private ArrayList<Bookmark>		bookmarks;
	private Bookmark				sel_bookmark;
	
	private static final String PluginName = "Bookmarks"; 
	private static final String PluginVersion = "0.2"; 
	
	public Bookmarks() {
		super();
		initialize();
	}
	
	private void initialize() {
		sel_bookmark = null;
		bookmarks = new ArrayList<Bookmark>();
		
		this.setLayout(new BorderLayout());
		this.add(getEditBMPanel(),BorderLayout.NORTH);
		this.add(getShowBMTabbedPane(),BorderLayout.CENTER);
	}
	
	private JPanel getEditBMPanel() {
		if(EditBMPanel == null) {
			EditBMPanel = new JPanel();
			EditBMPanel.setLayout(new GridLayout(0,2));
			
			EditBMPanel.add(new JLabel("URL:"));
			EditBMPanel.add(getEditBMURLTextField());
			
			EditBMPanel.add(new JLabel("Titel:"));
			EditBMPanel.add(getEditBMTitleTextField());
			
			EditBMPanel.add(new JLabel("Beschreibung:"));
			EditBMPanel.add(getEditBMDescrTextField());
			
			EditBMPanel.add(new JLabel("Tags (kommagetrennt):"));
			EditBMPanel.add(getEditBMTagsTextField());
			
			EditBMPanel.add(getEditBMAddBMButton());
			EditBMPanel.add(getEditBMEditButton());
		}
		return EditBMPanel;
	}
	
	private JTextField getEditBMURLTextField() {
		if(EditBMURLTextField == null) {
			EditBMURLTextField = new JTextField();
		}
		return EditBMURLTextField;
	}
	
	private JTextField getEditBMTitleTextField() {
		if(EditBMTitleTextField == null) {
			EditBMTitleTextField = new JTextField();
		}
		return EditBMTitleTextField;
	}
	
	private JTextField getEditBMDescrTextField() {
		if(EditBMDescrTextField == null) {
			EditBMDescrTextField = new JTextField();
		}
		return EditBMDescrTextField;
	}
	
	private JTextField getEditBMTagsTextField() {
		if(EditBMTagsTextField == null) {
			EditBMTagsTextField = new JTextField();
		}
		return EditBMTagsTextField;
	}
	
	private JButton getEditBMAddBMButton() {
		if(EditBMAddButton == null) {
			EditBMAddButton = new JButton();
			EditBMAddButton.setText("neues Bookmark...");
			EditBMAddButton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					getNewBMDialog().setVisible(true);
				}
			});
		}
		return EditBMAddButton;
	}
	
	private JButton getEditBMEditButton() {
		if(EditBMEditButton == null) {
			EditBMEditButton = new JButton();
			EditBMEditButton.setText("\u00C4nderungen speichern");
			EditBMEditButton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					if(sel_bookmark == null) return;
					Bookmark new_bookmark = new Bookmark(EditBMTitleTextField.getText(),
														EditBMDescrTextField.getText(),
														EditBMURLTextField.getText(),
														sel_bookmark.getDate(),
														EditBMTagsTextField.getText().split(","));
					try {
						peer.editBookmark(sel_bookmark, new_bookmark.getTitle(),
														new_bookmark.getDescription(),
														new_bookmark.getURL(),
														new_bookmark.getTags(),
														false); //TODO: recognize if BM is public or private
					} catch (RemoteException e1) {
						// cfg.log.addEntry("Remote-Exception: "+e1.getMessage(),"",false);
						cfg.log(e1);
						return;
					}
					  catch (IllegalArgumentException e1) { return; }
					
					bookmarks.set(bookmarks.indexOf(sel_bookmark),new_bookmark);
					refreshBookmarksByNameList();
				}
			});
		}
		return EditBMEditButton;
	}
	
	private JPanel getShowByBMPanel() {
		if(ShowByBMPanel == null) {
			ShowByBMPanel = new JPanel();
			ShowByBMPanel.setLayout(new BorderLayout());
			ShowByBMPanel.add(new JScrollPane(getShowByBMList()),BorderLayout.CENTER);
			ShowByBMPanel.add(getShowByBMControlPanel(),BorderLayout.SOUTH);
		}
		return ShowByBMPanel;
	}
	
	private JPanel getShowByBMControlPanel() {
		if(ShowByBMControlPanel == null) {
			ShowByBMControlPanel = new JPanel();
			ShowByBMControlPanel.add(getShowByBMRefreshButton());
			ShowByBMControlPanel.add(getShowByBMDeleteButton());
		}
		return ShowByBMControlPanel;
	}
	
	private JButton getShowByBMRefreshButton() {
		if(ShowByBMRefreshButton == null) {
			ShowByBMRefreshButton = new JButton();
			ShowByBMRefreshButton.setText("Liste aktualisieren");
			ShowByBMRefreshButton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					try {
						bookmarks = peer.getBookmarksAll();
					} catch (RemoteException ex) { cfg.log(ex); return; }
					refreshBookmarksByNameList();
				}
			});
		}
		return ShowByBMRefreshButton;
	}
	
	private JButton getShowByBMDeleteButton() {
		if(ShowByBMDeleteButton == null) {
			ShowByBMDeleteButton = new JButton();
			ShowByBMDeleteButton.setText("Eintrag l\u00F6schen");
			ShowByBMDeleteButton.setEnabled(false);
			ShowByBMDeleteButton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					PromptDeleteConfirm();
				}
			});
		}
		return ShowByBMDeleteButton;
	}
	
	private JTabbedPane getShowBMTabbedPane() {
		if(ShowBMTabbedPane == null) {
			ShowBMTabbedPane = new JTabbedPane();
			ShowBMTabbedPane.addTab("Nach Bookmarks sortieren",null,getShowByBMPanel(),"");
		}
		return ShowBMTabbedPane;
	}
	
	private JList getShowByBMList() {
		if(ShowByBMList == null) {
			ShowByBMList = new JList(getShowByBMListContent());
			ShowByBMList.addListSelectionListener(new ListSelectionListener() {
				public void valueChanged(ListSelectionEvent e) {
					int where = ShowByBMList.getSelectedIndex();
					if(where == -1) {
						ShowByBMDeleteButton.setEnabled(false);
						return;
					}
					ShowByBMDeleteButton.setEnabled(true);
					
					sel_bookmark = bookmarks.get(where);
					EditBMURLTextField.setText(sel_bookmark.getURL());
					EditBMURLTextField.select(0,0);
					EditBMTitleTextField.setText(sel_bookmark.getTitle());
					EditBMTitleTextField.select(0,0);
					EditBMDescrTextField.setText(sel_bookmark.getDescription());
					EditBMDescrTextField.select(0,0);
					
					//Tags als kommagetrennte Liste ausgeben
					String[] tags = sel_bookmark.getTags();
					String taglist = "";
					for(int i=0;i<tags.length;++i)
						taglist += tags[i]+(((i+1)<tags.length) ? "," : "");
					EditBMTagsTextField.setText(taglist);
					EditBMTagsTextField.select(0,0);
				}
			});
			ShowByBMList.addMouseListener(new MouseListener() {
				public void mouseClicked(MouseEvent e) {
					if(e.getClickCount() >= 2) {
						int where = ShowByBMList.getSelectedIndex();
						if(where == -1) return;
						try {
							Desktop.browse(new URL(bookmarks.get(where).getURL()));
						} catch (MalformedURLException e1) {
							cfg.log.addEntry("URL "+bookmarks.get(where).getURL()+" ist ung�ltig!","",false);
						}
						  catch (DesktopException e1) {
							cfg.log(e1);
						}
					}
				}

				public void mousePressed(MouseEvent e) { }
				public void mouseReleased(MouseEvent e) { }
				public void mouseEntered(MouseEvent e) { }
				public void mouseExited(MouseEvent e) { }
			});
			ShowByBMList.addKeyListener(new KeyListener() {

				public void keyTyped(KeyEvent e) { }
				public void keyPressed(KeyEvent e) { }
				public void keyReleased(KeyEvent e) {
					if(e.getKeyCode() == KeyEvent.VK_DELETE) {
						PromptDeleteConfirm();
					}
				}
			});
		}
		return ShowByBMList;
	}
	
	private void PromptDeleteConfirm() {
		if(ShowByBMList.getSelectedIndex() == -1) return;
		getDeleteConfirmInfoBMTitle().setText(sel_bookmark.getTitle());
		getDeleteConfirmInfoBMURL().setText(sel_bookmark.getURL());
		getDeleteConfirmDialog().setVisible(true);
	}
	
	private JDialog getDeleteConfirmDialog() {
		if(DeleteConfirmDialog == null) {
			DeleteConfirmDialog = new JDialog();
			DeleteConfirmDialog.setModal(true);
			DeleteConfirmDialog.setSize(320,180);
			DeleteConfirmDialog.setLayout(new BorderLayout());
			DeleteConfirmDialog.add(getDeleteConfirmInfoPanel(),BorderLayout.CENTER);
			DeleteConfirmDialog.add(getDeleteConfirmControlPanel(),BorderLayout.SOUTH);
		}
		return DeleteConfirmDialog;
	}
	
	private JPanel getDeleteConfirmInfoPanel() {
		if(DeleteConfirmInfoPanel == null) {
			DeleteConfirmInfoPanel = new JPanel();
			DeleteConfirmInfoPanel.setLayout(new GridLayout(0,1));
			DeleteConfirmInfoPanel.add(new JLabel("Wollen Sie wirklich das folgende Bookmark l�schen?"));
			DeleteConfirmInfoPanel.add(getDeleteConfirmInfoBMTitle());
			DeleteConfirmInfoPanel.add(getDeleteConfirmInfoBMURL());
		}
		return DeleteConfirmInfoPanel;
	}
	
	private JLabel getDeleteConfirmInfoBMTitle() {
		if(DeleteConfirmInfoBMTitle == null) {
			DeleteConfirmInfoBMTitle = new JLabel();
		}
		return DeleteConfirmInfoBMTitle;
	}
	
	private JLabel getDeleteConfirmInfoBMURL() {
		if(DeleteConfirmInfoBMURL == null) {
			DeleteConfirmInfoBMURL = new JLabel();
		}
		return DeleteConfirmInfoBMURL;
	}
	
	private JPanel getDeleteConfirmControlPanel() {
		if(DeleteConfirmControlPanel == null) {
			DeleteConfirmControlPanel = new JPanel();
			DeleteConfirmControlPanel.add(getDeleteConfirmControlDeleteButton());
			DeleteConfirmControlPanel.add(getDeleteConfirmControlDenyButton());
		}
		return DeleteConfirmControlPanel;
	}
	
	private JButton getDeleteConfirmControlDeleteButton() {
		if(DeleteConfirmControlDeleteButton == null) {
			DeleteConfirmControlDeleteButton = new JButton();
			DeleteConfirmControlDeleteButton.setText("L�schen");
			DeleteConfirmControlDeleteButton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					try {
						peer.deleteBookmark(sel_bookmark);
					} catch (RemoteException e1) { cfg.log(e1); return; }
					bookmarks.remove(sel_bookmark);
					sel_bookmark = null;
					refreshBookmarksByNameList();
					DeleteConfirmDialog.setVisible(false);
				}
			});
		}
		return DeleteConfirmControlDeleteButton;
	}
	
	private JButton getDeleteConfirmControlDenyButton() {
		if(DeleteConfirmControlDenyButton == null) {
			DeleteConfirmControlDenyButton = new JButton();
			DeleteConfirmControlDenyButton.setText("Nicht l�schen");
			DeleteConfirmControlDenyButton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					DeleteConfirmDialog.setVisible(false);
				}
			});
		}
		return DeleteConfirmControlDenyButton;
	}
	
	private void ClearNewBMTextFields() {
		getNewBMURLTextField().setText("");
		getNewBMTitleTextField().setText("");
		getNewBMDescrTextField().setText("");
		getNewBMTagsTextField().setText("");
		getNewBMPublicCheckBox().setSelected(true);
	}
	
	private JDialog getNewBMDialog() {
		if(NewBMDialog == null) {
			NewBMDialog = new JDialog();
			NewBMDialog.setLayout(new BorderLayout());
			NewBMDialog.setSize(500,150);
			NewBMDialog.setModal(true);
			NewBMDialog.add(getNewBMEditPanel(),BorderLayout.CENTER);
			NewBMDialog.add(getNewBMControlPanel(),BorderLayout.SOUTH);
		}
		return NewBMDialog;
	}
	
	private JPanel getNewBMEditPanel() {
		if(NewBMEditPanel == null) {
			NewBMEditPanel = new JPanel();
			NewBMEditPanel.setLayout(new GridLayout(0,2));
			
			NewBMEditPanel.add(new JLabel("URL:"));
			NewBMEditPanel.add(getNewBMURLTextField());
			
			NewBMEditPanel.add(new JLabel("Titel:"));
			NewBMEditPanel.add(getNewBMTitleTextField());
			
			NewBMEditPanel.add(new JLabel("Beschreibung:"));
			NewBMEditPanel.add(getNewBMDescrTextField());
			
			NewBMEditPanel.add(new JLabel("Tags (kommagetrennt):"));
			NewBMEditPanel.add(getNewBMTagsTextField());
			
			NewBMEditPanel.add(getNewBMPublicCheckBox());
		}
		return NewBMEditPanel;
	}
	
	private JTextField getNewBMURLTextField() {
		if(NewBMURLTextField == null) {
			NewBMURLTextField = new JTextField();
		}
		return NewBMURLTextField;
	}
	
	private JTextField getNewBMTitleTextField() {
		if(NewBMTitleTextField == null) {
			NewBMTitleTextField = new JTextField();
		}
		return NewBMTitleTextField;
	}
	
	private JTextField getNewBMDescrTextField() {
		if(NewBMDescrTextField == null) {
			NewBMDescrTextField = new JTextField();
		}
		return NewBMDescrTextField;
	}
	
	private JTextField getNewBMTagsTextField() {
		if(NewBMTagsTextField == null) {
			NewBMTagsTextField = new JTextField();
		}
		return NewBMTagsTextField;
	}
	
	private JCheckBox getNewBMPublicCheckBox() {
		if(NewBMPublicCheckBox == null) {
			NewBMPublicCheckBox = new JCheckBox();
			NewBMPublicCheckBox.setText("�ffentlich?");
			NewBMPublicCheckBox.addItemListener(new ItemListener() {
				public void itemStateChanged(ItemEvent e) {
					if(e.getStateChange() == ItemEvent.SELECTED)
						NewBMPublicCheckBoxState = true;
					else
						NewBMPublicCheckBoxState = false;
				}
			});
			NewBMPublicCheckBox.setSelected(true);
		}
		return NewBMPublicCheckBox;
	}
	
	private JPanel getNewBMControlPanel() {
		if(NewBMControlPanel == null) {
			NewBMControlPanel = new JPanel();
			NewBMControlPanel.add(getNewBMSaveButton());
			NewBMControlPanel.add(getNewBMCancelButton());
		}
		return NewBMControlPanel;
	}
	
	private JButton getNewBMSaveButton() {
		if(NewBMSaveButton == null) {
			NewBMSaveButton = new JButton();
			NewBMSaveButton.setText("Speichern");
			NewBMSaveButton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					Date now = new Date(System.currentTimeMillis());
					String bmdate = new SimpleDateFormat("yyyy-MM-dd").format(now)+"T"+(new SimpleDateFormat("HH:mm:ss")).format(now)+"Z";
					Bookmark new_bookmark = new Bookmark(NewBMTitleTextField.getText(),
														 NewBMDescrTextField.getText(),
														 NewBMURLTextField.getText(),
														 bmdate,
														 NewBMTagsTextField.getText().split(","));
					boolean isPublic = NewBMPublicCheckBoxState;
					
					if(	new_bookmark.getURL().equals("") ||
						new_bookmark.getTitle().equals("")
					  )
						return;
					
					try {
						peer.addBookmark(new_bookmark,isPublic);
					} catch (RemoteException e1) {
						cfg.log(e1);
						return;
					}
					
					bookmarks.add(new_bookmark);
					refreshBookmarksByNameList();
					
					ClearNewBMTextFields();
					NewBMDialog.setVisible(false);
				}
			});
		}
		return NewBMSaveButton;
	}
	
	private JButton getNewBMCancelButton() {
		if(NewBMCancelButton == null) {
			NewBMCancelButton = new JButton();
			NewBMCancelButton.setText("Verwerfen");
			NewBMCancelButton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					ClearNewBMTextFields();
					NewBMDialog.setVisible(false);
				}
			});
		}
		return NewBMCancelButton;
	}
	
	private DefaultListModel getShowByBMListContent() {
		if(ShowByBMListContent == null)
			ShowByBMListContent = new DefaultListModel();
		return ShowByBMListContent;
	}
	
	private void refreshBookmarksByNameList() {
			if(bookmarks == null) return;
			ShowByBMListContent.clear();
			for(int i=0;i<bookmarks.size(); i++)
				ShowByBMListContent.addElement(bookmarks.get(i).getTitle());
	}
	
	public void init(Peer peer, Config cfg) {
		this.peer = peer;
		this.cfg = cfg;
		
		try {
			this.bookmarks = this.peer.getBookmarksAll();
		} catch (RemoteException e) { cfg.log(e); return; }
		
		refreshBookmarksByNameList();
	}

	public String getDescription() { return PluginName; }
	public String getMenuPath() { return PanelPlugin.ADMIN; }
	public ImageIcon getMenuIcon() { return ICON_BOOKMARKS; }
	public String getMenuToolTip() { return "Dient der Anzeige und Verwaltung der Bookmarks"; }
	public int getUpdateDelay() { return 0; }
	public String getVersion() { return PluginVersion; }
	public int getNeededYacyRevision() { return 2915; }
	public void doUpdate() { }
}
