/* 
 * Autor: Franz Brauße
 * License: GPL v2 
 */

package plugins;

import general.Config;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.io.File;
import java.io.IOException;

import javax.swing.event.TreeExpansionEvent;
import javax.swing.event.TreeExpansionListener;
import javax.swing.event.TreeWillExpandListener;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.TreePath;
import javax.swing.BorderFactory;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTree;
import javax.swing.SwingConstants;

import java.rmi.RemoteException;
import java.util.ArrayList;


import javax.swing.JToggleButton;

import views.PanelPlugin;
import yacy.Peer;
import yacy.datatypes.ShareElement;
import yacy.datatypes.ShareFolder;
import yacy.event.PeerEvent;
import yacy.event.PeerEventListener;
import yacy.event.PeerInterruptableEvent;
import yacy.event.PeerTransferFinishedEvent;

public class FileShare extends JPanel implements views.PanelPlugin {
	
	private static final long	serialVersionUID	= 1L;
	
	private JPanel 		jPanel2 			= null;
	private JSplitPane 	jSplitPane 			= null;
	private JPanel 		jPRemote 			= null;
	private JPanel 		jpLocal 			= null;
	private JLabel 		jLabel 				= null;
	private JLabel 		jLabel1 			= null;
	private JPanel 		jPRemoteToolbar 	= null;
	private JButton 	btnDeleteRemote 	= null;
	private JButton 	btnTransRemoteLocal = null;
	private JButton 	btnEditComment 		= null;
	private JPanel 		jPLocalToolbar 		= null;
	private JButton 	btnTransLocalRemote = null;
	private JButton 	btnDeleteLocal 		= null;
	private JPanel 		jPanel 				= null;
	private JList 		dList 				= null;
	private JComboBox 	jCBLocal 			= null;
	private JComboBox 	jCBRemote 			= null;
	private JScrollPane	jScrollPaneLocal 	= null;
	private JScrollPane jScrollPaneRemote 	= null;
	
	private static final String PluginName 			= "FileShare";
	private static final String PluginVersion 		= "0.1";
	private static final int 	PluginUpdateDelay 	= 0;	// no continous updates needed
	
	private Peer 	peer 	= null;
	private Config 	cfg 	= null;
	private JTree 	local 	= null;
	private JTree 	remote 	= null;
	
	private DefaultMutableTreeNode 	rpath 				= null;
	private DefaultMutableTreeNode 	lpath 				= null;
	private TreePath 				currentlyOpening 	= null;
	private JPanel 					jPanel1 			= null;
	private JLabel 					jLabel2 			= null;
	private JProgressBar 			jProgressBar 		= null;
	private JToggleButton 			jTBIndexFiles 		= null;
	private Thread 					transferRunning 	= null;
	
	private static final File[] roots;
	static {
		// find roots / drives
		ArrayList<File> r = new ArrayList<File>();
		for (int i=0; i<File.listRoots().length; i++) {
			if (File.listRoots()[i].exists())
				r.add(File.listRoots()[i]);
		}
		roots = (r.size() > 0) ? r.toArray(new File[r.size()]) : new File[] { new File("/") };
	}
	
	public final String getDescription() 		{ return PluginName; }
	public String 		getVersion() 			{ return PluginVersion; }
	public String 		getMenuPath() 			{ return PanelPlugin.ADMIN; }
	public int 			getUpdateDelay() 		{ return PluginUpdateDelay; }
	public int 			getNeededYacyRevision() { return 2857; }
	public ImageIcon 	getMenuIcon() 			{ return new ImageIcon(); }
	public String		getMenuToolTip() 		{ return ""; }
	public void 		doUpdate() 				{  }
	
	public void init(Peer peer, Config cfg) {
		this.peer = peer;
		this.peer.registerPeerEventListener(new PeerEventListener() {
			public void peerEvent(PeerEvent e) {
				if (e instanceof PeerTransferFinishedEvent) {
					transferRunning = null;
					refreshRemote();
				}
			}
			public void peerInterruptableEvent(PeerInterruptableEvent e) {
				if (transferRunning != null)
					e.interrupt("Transfer running", transferRunning);
			}
		});
		this.cfg = cfg;
		if (super.getWidth() > 0) {
			jSplitPane.setDividerLocation(super.getWidth() / 2);
			this.updateUI();
		}
		refreshLocal();
		refreshRemote();
	}
	
	private void refreshLocal() {
		refreshLocal(roots[0].getAbsolutePath());
	}
	
	private void refreshLocal(String dir) {
		if (!new File(dir).exists()) return;
		if (lpath != null && lpath.toString().equals(dir)) return;
		lpath = getLocalDir(dir);
		local = null;
		local = getLocal();
		jScrollPaneLocal.setViewportView(local);
		jSplitPane.updateUI();
	}
	
	private void refreshRemote() {
		refreshRemote("/");
	}
	
	private void refreshRemote(String dir) {
		rpath = getRemoteDir(dir);
		remote = null;
		remote = getRemote();
		jScrollPaneRemote.setViewportView(remote);
		jSplitPane.updateUI();
	}
	
	private DefaultMutableTreeNode[] getRemoteDirContents(String path) {
		ArrayList<DefaultMutableTreeNode> r = new ArrayList<DefaultMutableTreeNode>();
		try {
			for (ShareElement d : peer.getShareDirList(path)) {
				DefaultMutableTreeNode f = new DefaultMutableTreeNode(d.getName());
				if (d instanceof ShareFolder)
					f.add(new DefaultMutableTreeNode("dummy"));		// to display it as a folder
				r.add(f);
			}
		} catch (RemoteException e) { cfg.log(e); return new DefaultMutableTreeNode[] {  }; }
		return r.toArray(new DefaultMutableTreeNode[r.size()]);
	}
	
	private DefaultMutableTreeNode getRemoteDir(String path) {
		DefaultMutableTreeNode r = new DefaultMutableTreeNode();
		try {
			for (ShareElement d : peer.getShareDirList(path)) {
				DefaultMutableTreeNode f = new DefaultMutableTreeNode(d.getName());
				if (d instanceof ShareFolder)
					f.add(new DefaultMutableTreeNode("dummy"));
				r.add(f);
			}
		} catch (RemoteException e) { cfg.log(e); }
		return r;
	}
	
	private DefaultMutableTreeNode[] getLocalDirContents(String path) {		
		File p = new File(path);
		ArrayList<DefaultMutableTreeNode> r = new ArrayList<DefaultMutableTreeNode>();
		if (!p.isDirectory() || !p.canRead())
			return new DefaultMutableTreeNode[] {  };
		
		boolean separate = path.length() > 1 && !path.endsWith(File.separator);
		for (String d : p.list()) {
			DefaultMutableTreeNode akt = new DefaultMutableTreeNode(d);
			if (new File(path + (separate ? File.separator : "") + d ).isDirectory())
				akt.add(new DefaultMutableTreeNode("dummy"));
			r.add(akt);
		}
		return r.toArray(new DefaultMutableTreeNode[r.size()]);
	}
	
	private DefaultMutableTreeNode getLocalDir(String path) {
		DefaultMutableTreeNode r = new DefaultMutableTreeNode(path);
		DefaultMutableTreeNode[] d = getLocalDirContents(path);
		for (int i=0; i<d.length; i++)
			r.add(d[i]);
		return r;
	}

	private String tree2String(TreePath d) {
		if (d == null) return "/";
		String r = "";
		for (int i=1; i<d.getPathCount(); i++)
			r += "/" + d.getPath()[i].toString();
		return r;
	}
	
	public FileShare() {
		super();
		initialize();
	}
	
	private void initialize() {
		this.setSize(551, 498);
		this.setLayout(new BorderLayout());
		this.add(getJPanel2(), BorderLayout.CENTER);
		this.add(getJPanel(), BorderLayout.SOUTH);
	}

	private JPanel getJPanel2() {
		if (jPanel2 == null) {
			GridBagConstraints gridBagConstraints41 = new GridBagConstraints();
			gridBagConstraints41.gridx = 0;
			gridBagConstraints41.fill = GridBagConstraints.HORIZONTAL;
			gridBagConstraints41.gridy = 1;
			GridBagConstraints gridBagConstraints = new GridBagConstraints();
			gridBagConstraints.fill = GridBagConstraints.BOTH;
			gridBagConstraints.gridy = 0;
			gridBagConstraints.weightx = 1.0;
			gridBagConstraints.weighty = 1.0;
			gridBagConstraints.insets = new Insets(5, 5, 5, 5);
			gridBagConstraints.gridx = 0;
			jPanel2 = new JPanel();
			jPanel2.setLayout(new GridBagLayout());
			jPanel2.add(getJSplitPane(), gridBagConstraints);
			jPanel2.add(getJPanel1(), gridBagConstraints41);
		}
		return jPanel2;
	}

	private JSplitPane getJSplitPane() {
		if (jSplitPane == null) {
			jSplitPane = new JSplitPane();
			jSplitPane.setDividerLocation(255);
			jSplitPane.setDividerSize(8);
			jSplitPane.setBorder(BorderFactory.createEmptyBorder(0, 0, 0, 0));
			jSplitPane.setLeftComponent(getJpLocal());
			jSplitPane.setRightComponent(getJPRemote());
		}
		return jSplitPane;
	}

	private JPanel getJPRemote() {
		if (jPRemote == null) {
			GridBagConstraints gridBagConstraints1 = new GridBagConstraints();
			gridBagConstraints1.fill = GridBagConstraints.BOTH;
			gridBagConstraints1.gridy = 4;
			gridBagConstraints1.weightx = 1.0;
			gridBagConstraints1.weighty = 1.0;
			gridBagConstraints1.gridwidth = 2;
			gridBagConstraints1.gridx = 0;
			GridBagConstraints gridBagConstraints8 = new GridBagConstraints();
			gridBagConstraints8.fill = GridBagConstraints.HORIZONTAL;
			gridBagConstraints8.gridy = 3;
			gridBagConstraints8.weightx = 1.0;
			gridBagConstraints8.gridwidth = 2;
			gridBagConstraints8.insets = new Insets(3, 5, 3, 3);
			gridBagConstraints8.gridx = 0;
			GridBagConstraints gridBagConstraints5 = new GridBagConstraints();
			gridBagConstraints5.gridx = 0;
			gridBagConstraints5.fill = GridBagConstraints.VERTICAL;
			gridBagConstraints5.insets = new Insets(3, 2, 0, 5);
			gridBagConstraints5.anchor = GridBagConstraints.WEST;
			gridBagConstraints5.gridy = 2;
			GridBagConstraints gridBagConstraints3 = new GridBagConstraints();
			gridBagConstraints3.gridx = 1;
			gridBagConstraints3.anchor = GridBagConstraints.EAST;
			gridBagConstraints3.insets = new Insets(3, 5, 3, 5);
			gridBagConstraints3.gridy = 2;
			jLabel = new JLabel();
			jLabel.setText("Remote");
			jPRemote = new JPanel();
			jPRemote.setLayout(new GridBagLayout());
			jPRemote.add(jLabel, gridBagConstraints3);
			jPRemote.add(getJPRemoteToolbar(), gridBagConstraints5);
			jPRemote.add(getJCBRemote(), gridBagConstraints8);
			jPRemote.add(getJScrollPaneRemote(), gridBagConstraints1);
		}
		return jPRemote;
	}

	private JPanel getJpLocal() {
		if (jpLocal == null) {
			GridBagConstraints gridBagConstraints9 = new GridBagConstraints();
			gridBagConstraints9.fill = GridBagConstraints.BOTH;
			gridBagConstraints9.weighty = 1.0;
			gridBagConstraints9.gridy = 3;
			gridBagConstraints9.gridx = 1;
			gridBagConstraints9.gridwidth = 2;
			gridBagConstraints9.weightx = 1.0;
			GridBagConstraints gridBagConstraints7 = new GridBagConstraints();
			gridBagConstraints7.fill = GridBagConstraints.HORIZONTAL;
			gridBagConstraints7.gridy = 2;
			gridBagConstraints7.weightx = 1.0;
			gridBagConstraints7.gridwidth = 2;
			gridBagConstraints7.insets = new Insets(3, 3, 3, 5);
			gridBagConstraints7.gridx = 1;
			GridBagConstraints gridBagConstraints6 = new GridBagConstraints();
			gridBagConstraints6.gridx = 2;
			gridBagConstraints6.anchor = GridBagConstraints.EAST;
			gridBagConstraints6.insets = new Insets(3, 5, 0, 2);
			gridBagConstraints6.gridy = 0;
			GridBagConstraints gridBagConstraints4 = new GridBagConstraints();
			gridBagConstraints4.gridx = 1;
			gridBagConstraints4.anchor = GridBagConstraints.WEST;
			gridBagConstraints4.insets = new Insets(3, 5, 3, 5);
			gridBagConstraints4.gridy = 0;
			jLabel1 = new JLabel();
			jLabel1.setText("Local");
			jpLocal = new JPanel();
			jpLocal.setLayout(new GridBagLayout());
			jpLocal.add(jLabel1, gridBagConstraints4);
			jpLocal.add(getJPLocalToolbar(), gridBagConstraints6);
			jpLocal.add(getJCBLocal(), gridBagConstraints7);
			jpLocal.add(getJScrollPaneLocal(), gridBagConstraints9);
		}
		return jpLocal;
	}

	private JPanel getJPRemoteToolbar() {
		if (jPRemoteToolbar == null) {
			FlowLayout flowLayout = new FlowLayout();
			flowLayout.setHgap(3);
			flowLayout.setAlignment(FlowLayout.LEFT);
			flowLayout.setVgap(0);
			jPRemoteToolbar = new JPanel();
			jPRemoteToolbar.setMinimumSize(new Dimension(90, 24));
			jPRemoteToolbar.setMaximumSize(new Dimension(32767, 24));
			jPRemoteToolbar.setLayout(flowLayout);
			jPRemoteToolbar.setPreferredSize(new Dimension(160, 24));
			jPRemoteToolbar.add(getBtnTransRemoteLocal(), null);
			jPRemoteToolbar.add(getBtnDeleteRemote(), null);
			jPRemoteToolbar.add(getBtnEditComment(), null);
		}
		return jPRemoteToolbar;
	}
	
	private boolean confirm(String question, String title) {
		return (JOptionPane.showConfirmDialog(
				this,
				question,
				title,
				JOptionPane.YES_NO_OPTION) == 0);
	}

	private JButton getBtnDeleteRemote() {
		if (btnDeleteRemote == null) {
			btnDeleteRemote = new JButton();
			btnDeleteRemote.setPreferredSize(new Dimension(24, 24));
			btnDeleteRemote.setToolTipText("Gewählte Datei(en) löschen");
			btnDeleteRemote.setIcon(new ImageIcon("ressources/fileclose.png"));
			btnDeleteRemote.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					if (remote.getSelectionCount() == 0) return;
					if (!confirm("Möchten Sie die Dateien wirklich löschen?", "Dateien löschen")) return;
					new Thread() {
						public void run() {
							for (TreePath selpath : remote.getSelectionPaths()) {
								cfg.log.addEntry("Deleting remote share: " + tree2String(selpath) + " ...", this.toString(), true);
								try {
									peer.deleteShareFile(tree2String(selpath));
								} catch (RemoteException ee) { cfg.log(ee); }
								refreshRemote();
								cfg.log.addEntry("Deletion of " + tree2String(selpath) + " finished", this.toString(), false);
							}
						}
					}.start();
				}
			});
		}
		return btnDeleteRemote;
	}

	private JButton getBtnTransRemoteLocal() {
		if (btnTransRemoteLocal == null) {
			btnTransRemoteLocal = new JButton();
			btnTransRemoteLocal.setIcon(new ImageIcon("ressources/2leftarrow.png"));
			btnTransRemoteLocal.setMnemonic(KeyEvent.VK_UNDEFINED);
			btnTransRemoteLocal.setToolTipText("Gewählte Datei(en) herunterladen");
			btnTransRemoteLocal.setPreferredSize(new Dimension(24, 24));
			btnTransRemoteLocal.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					System.out.println("actionPerformed()"); // TODO Auto-generated Event stub actionPerformed()
				}
			});
		}
		return btnTransRemoteLocal;
	}

	private JButton getBtnEditComment() {
		if (btnEditComment == null) {
			btnEditComment = new JButton();
			btnEditComment.setPreferredSize(new Dimension(24, 24));
			btnEditComment.setToolTipText("Kommentar der Datei editieren");
			btnEditComment.setIcon(new ImageIcon("ressources/text_left.png"));
			btnEditComment.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					if (remote.getSelectionCount() != 1)
						return;
					
					String newComment = JOptionPane.showInputDialog("Neuer Kommentar");
					boolean indexFile = JOptionPane.showConfirmDialog(
							null,
							"Soll die Datei und der neue Kommentar in den Index aufgenommen werden?",
							"Datei indexieren?",
							JOptionPane.YES_NO_OPTION) == 0;
					
					try {
						peer.editShareFileComment(tree2String(remote.getSelectionPath()), newComment, indexFile);
					} catch (RemoteException ee) { cfg.log(ee); }
				}
			});
		}
		return btnEditComment;
	}

	private JPanel getJPLocalToolbar() {
		if (jPLocalToolbar == null) {
			FlowLayout flowLayout1 = new FlowLayout();
			flowLayout1.setAlignment(FlowLayout.RIGHT);
			flowLayout1.setVgap(0);
			flowLayout1.setHgap(3);
			jPLocalToolbar = new JPanel();
			jPLocalToolbar.setMinimumSize(new Dimension(109, 24));
			jPLocalToolbar.setMaximumSize(new Dimension(32767, 24));
			jPLocalToolbar.setLayout(flowLayout1);
			jPLocalToolbar.setPreferredSize(new Dimension(160, 24));
			jPLocalToolbar.add(getJTBIndexFiles(), null);
			jPLocalToolbar.add(getBtnDeleteLocal(), null);
			jPLocalToolbar.add(getBtnTransLocalRemote(), null);
		}
		return jPLocalToolbar;
	}

	private JButton getBtnTransLocalRemote() {
		if (btnTransLocalRemote == null) {
			btnTransLocalRemote = new JButton();
			btnTransLocalRemote.setPreferredSize(new Dimension(24, 24));
			btnTransLocalRemote.setToolTipText("Gewählte Datei(en) zu Peer übertragen");
			btnTransLocalRemote.setIcon(new ImageIcon("ressources/2rightarrow.png"));
			btnTransLocalRemote.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					if (local.getSelectionCount() == 0) return;
					File d = null;
					String path = null;
					
					for (TreePath selpath : local.getSelectionPaths()) {
						d = new File(tree2String(selpath));
						path = "/";
						if (remote.getSelectionCount() > 0 &&
								!((DefaultMutableTreeNode)remote.getSelectionPath().getLastPathComponent()).isLeaf())
							path = tree2String(remote.getSelectionPath());
						
						try {
							transferRunning = peer.uploadShareFile(d, path, "", jTBIndexFiles.isSelected());
						} catch (IOException e1) { cfg.log(e1); }	
					}
				}
			});
		}
		return btnTransLocalRemote;
	}

	private JButton getBtnDeleteLocal() {
		if (btnDeleteLocal == null) {
			btnDeleteLocal = new JButton();
			btnDeleteLocal.setPreferredSize(new Dimension(24, 24));
			btnDeleteLocal.setToolTipText("Gewählte Datei(en) löschen");
			btnDeleteLocal.setIcon(new ImageIcon("ressources/fileclose.png"));
			btnDeleteLocal.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					if (!confirm("Möchten Sie die Dateien wirklich löschen?", "Dateien löschen"))
						return;
					if (local.getSelectionCount() == 0)
						return;
					for (TreePath selpath : local.getSelectionPaths()) {
						new File(tree2String(selpath)).delete();
						refreshLocal();
					}
				}
			});
		}
		return btnDeleteLocal;
	}
	
	private JPanel getJPanel() {
		if (jPanel == null) {
			jPanel = new JPanel();
			jPanel.setLayout(new BorderLayout());
			jPanel.add(getDList(), java.awt.BorderLayout.CENTER);
		}
		return jPanel;
	}

	private JList getDList() {
		if (dList == null) {
			dList = new JList();
			dList.setPreferredSize(new Dimension(0, 80));
			dList.setVisible(false);
		}
		return dList;
	}
	
	private JComboBox getJCBLocal() {
		if (jCBLocal == null) {
			jCBLocal = new JComboBox();
			jCBLocal.setEditable(true);
			jCBLocal.setMinimumSize(new Dimension(32, 19));
			jCBLocal.setMaximumSize(new Dimension(32767, 19));
			jCBLocal.setPreferredSize(new Dimension(32, 19));
			for (int i=0; i<roots.length; i++)
				jCBLocal.addItem(roots[i]);
			jCBLocal.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					refreshLocal(jCBLocal.getSelectedItem().toString());
				}
			});
		}
		return jCBLocal;
	}
	
	private JComboBox getJCBRemote() {
		if (jCBRemote == null) {
			jCBRemote = new JComboBox();
			jCBRemote.setEditable(true);
			jCBRemote.setMinimumSize(new Dimension(32, 19));
			jCBRemote.setMaximumSize(new Dimension(32767, 19));
			jCBRemote.setPreferredSize(new Dimension(32, 19));
			jCBRemote.addItem("/");
			jCBRemote.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					refreshRemote(jCBRemote.getSelectedItem().toString());
				}
			});
		}
		return jCBRemote;
	}
	
	private JScrollPane getJScrollPaneLocal() {
		if (jScrollPaneLocal == null) {
			jScrollPaneLocal = new JScrollPane();
			jScrollPaneLocal.setViewportView(getLocal());
		}
		return jScrollPaneLocal;
	}
	
	private JTree getLocal() {
		JTree r = new JTree(lpath);
		r.setRootVisible(false);
		r.setShowsRootHandles(true);
		r.addTreeExpansionListener(new TreeExpansionListener() {
			public void treeExpanded(TreeExpansionEvent e) {
				TreePath where = e.getPath();
				if (where == currentlyOpening || where.getPath()[where.getPath().length-1].toString() == null)
					currentlyOpening = null;
				jProgressBar.setVisible(false);
				return;
			}
			public void treeCollapsed(TreeExpansionEvent e) {  }
		});
		r.addTreeWillExpandListener(new TreeWillExpandListener() {
			public void treeWillExpand(TreeExpansionEvent e) {
				TreePath where = e.getPath();
				if (where == currentlyOpening || where.getPath()[where.getPath().length-1].toString() == null)
					return;
				else
					currentlyOpening = e.getPath();
				
				jProgressBar.setVisible(true);
				jProgressBar.updateUI();
				jPanel1.updateUI();
				((DefaultMutableTreeNode)where.getLastPathComponent()).removeAllChildren();
				DefaultMutableTreeNode[] listing = getLocalDirContents(tree2String(where));
				for (int h=0; h<listing.length; h++)
					((DefaultMutableTreeNode)where.getLastPathComponent()).add(listing[h]);
			}
			public void treeWillCollapse(TreeExpansionEvent e) {  }
		});
		return r;
	}
	
	private JScrollPane getJScrollPaneRemote() {
		if (jScrollPaneRemote == null) {
			jScrollPaneRemote = new JScrollPane();
			jScrollPaneRemote.setViewportView(getRemote());
		}
		return jScrollPaneRemote;
	}
	
	/*
	private class RNode extends DefaultMutableTreeNode {
		
		private boolean isUpdated = false;
		private boolean isDirectory = false;
		private String nodeName = null;
		
		public RNode(Object name) { this.nodeName = name.toString(); }
		
		public void doUpdate(DefaultMutableTreeNode[] contents) {
			if (!this.isUpdated && this.getChildCount() > 0 && !this.isDirectory)
				for (int i=0; i<contents.length; this.add(contents[i++])) ;
			this.isUpdated = true;
		}
		
		public boolean isDirectory() { return this.isDirectory; }
		
		public boolean isUpdated() { return this.isUpdated; }
		
		public void setDirectory(boolean yes) {
			if (yes)
				this.add(new DefaultMutableTreeNode("dummy"));
			this.isDirectory = yes;
		}
		
		public String toString() { return this.nodeName; }
	}
	*/
	
	private JTree getRemote() {
		JTree r = new JTree(rpath);
		r.setRootVisible(false);
		r.setShowsRootHandles(true);
		r.addTreeExpansionListener(new TreeExpansionListener() {
			public void treeExpanded(TreeExpansionEvent e) {
				TreePath where = e.getPath();
				if (where == currentlyOpening || where.getPath()[1].toString() == null)
					currentlyOpening = null;
				jProgressBar.setVisible(false);
				return;
			}
			public void treeCollapsed(TreeExpansionEvent e) {  }
		});
		r.addTreeWillExpandListener(new TreeWillExpandListener() {
			public void treeWillExpand(TreeExpansionEvent e) {
				TreePath where = e.getPath();
				if (where == currentlyOpening || where.getPath()[1].toString() == null)
					return;
				else
					currentlyOpening = e.getPath();
				
				jProgressBar.setVisible(true);
				jPanel1.updateUI();
				((DefaultMutableTreeNode)where.getLastPathComponent()).removeAllChildren();
				DefaultMutableTreeNode[] listing = getRemoteDirContents(tree2String(where));
				for (int h=0; h<listing.length; h++)
					((DefaultMutableTreeNode)where.getLastPathComponent()).add(listing[h]);
			}
			public void treeWillCollapse(TreeExpansionEvent e) {  }
		});
		return r;
	}
	
	private JPanel getJPanel1() {
		if (jPanel1 == null) {
			jLabel2 = new JLabel("");
			jLabel2.setPreferredSize(new Dimension(38, 21));
			jLabel2.setHorizontalAlignment(SwingConstants.LEFT);
			jPanel1 = new JPanel();
			jPanel1.setLayout(new BorderLayout());
			jPanel1.add(jLabel2, BorderLayout.CENTER);
			jPanel1.add(getJProgressBar(), BorderLayout.EAST);
		}
		return jPanel1;
	}
	
	private JProgressBar getJProgressBar() {
		if (jProgressBar == null) {
			jProgressBar = new JProgressBar();
			jProgressBar.setString("");
			jProgressBar.setPreferredSize(new Dimension(60, 14));
			jProgressBar.setVisible(false);
			jProgressBar.setIndeterminate(true);
		}
		return jProgressBar;
	}
	
	private JToggleButton getJTBIndexFiles() {
		if (jTBIndexFiles == null) {
			jTBIndexFiles = new JToggleButton();
			jTBIndexFiles.setPreferredSize(new Dimension(24, 24));
			jTBIndexFiles.setMaximumSize(new Dimension(24, 24));
			jTBIndexFiles.setToolTipText("Datei(en) beim Hochladen indexieren");
			jTBIndexFiles.setMinimumSize(new Dimension(24, 24));
		}
		return jTBIndexFiles;
	}
	
}
