/* 
 * Autor: Franz Brauße
 * License: GPL v2 
 */

package plugins;

import general.Config;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.HashMap;

import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JProgressBar;

import tools.DGramm;
import views.PanelPlugin;

import yacy.Peer;


public class Status extends JPanel implements views.PanelPlugin {
	
	private static final long serialVersionUID = 1L;
	
	private JPanel 			jPanel 		= null;
	private JLabel 			jLabel 		= null;
	private JLabel 			links 		= null;
	private JLabel 			jLabel2 	= null;
	private JLabel 			words 		= null;
	private JLabel 			jLabel4 	= null;
	private JLabel 			jLabel5 	= null;
	private JProgressBar 	memory 		= null;
	private JLabel 			jLabel6 	= null;
	private JLabel 			pStatus 	= null;
	private JLabel 			ppm 		= null;
	private JLabel 			jLabel11 	= null;
	private JProgressBar 	ramCache 	= null;
	private JLabel 			title 		= null;
	private JPanel 			jPanel2 	= null;
	private JLabel 			peerCount 	= null;
	private JLabel 			linkCount 	= null;
	private JLabel 			wordCount 	= null;
	private DGramm 			jPanel4 	= null;
	private JLabel 			aktMem 		= null;
	private JLabel 			aktRamCache = null;
	private JLabel 			jLabel1 	= null;
	private JLabel 			peerStatus 	= null;
	private JPopupMenu 		ppmMenu 	= null;
	
	private static final String 					PluginName = "Peer Status"; 
	private static final String 					PluginVersion = "0.1"; 
	private static final HashMap<String,String[]> 	pTypes = new HashMap<String,String[]>();
	private static final ArrayList<String> 			pStati = new ArrayList<String>();
	private static final ArrayList<Color>			pStatiColors = new ArrayList<Color>();
	static {
		pTypes.put(Peer.STATE_VIRGIN, new String[] { "Virgin",
				"Der Peer konnte noch keine Verbindung mit dem YaCy-Netzwerk herstellen." });
		pTypes.put(Peer.STATE_JUNIOR, new String[] { "Junior",
				"Der Peer ist mit dem YaCy-Netzwerk verbunden, kann von außen allerdings\n" +
				"nicht erreicht werden. Dies kann zum Beispiel Firewall-Einstellungen oder\n" +
				"Remote-Proxies liegen." });
		pTypes.put(Peer.STATE_SENIOR, new String[] { "Senior",
				"Der Peer ist mit dem YaCy-Netzwerk verbunden und trägt zum globalen Index bei." });
		pTypes.put(Peer.STATE_PRINCIPAL, new String[] { "Prinicipal",
				"Der Peer ist mit dem YaCy-Netzwerk verbunden und trägt zum globalen Index bei.\n" + 
				"Außerdem hilft er anderen Peers dabei, das Netz zu finden." });
		pStati.add("Online");
		pStati.add("No Internet available");
		pStati.add("Not online");
		pStatiColors.add(Color.GREEN);
		pStatiColors.add(Color.ORANGE);
		pStatiColors.add(Color.RED);
	}

	private static final Color NORMAL = Color.BLACK;
	private static final Color WARNING = Color.RED;
	
	private static final Dimension mid_dim1 = new Dimension(150,50);
	private static final Dimension mid_dim = new Dimension(150,14);
	
	private static final double memWarn = .85D;

	private int 	maxMemory = 0;
	private int 	maxRamCache = 0;
	private Peer 	peer = null;
	private Config 	cfg = null;
	
	public Status() {
		super();
		initialize();
		this.repaint();
	}
	
	private void initialize() {
		BorderLayout borderLayout = new BorderLayout();
		borderLayout.setVgap(5);
		borderLayout.setHgap(0);
		this.setLayout(borderLayout);
		this.setSize(501, 409);
		this.add(getJPanel(), BorderLayout.CENTER);
		this.add(getJPanel2(), BorderLayout.SOUTH);
	}
	
	public ImageIcon 	getMenuIcon() 			{ return new ImageIcon(); }
	public String		getMenuToolTip() 		{ return null; }
	public String 		getVersion() 			{ return PluginVersion; }
	public String 		getDescription() 		{ return PluginName; }
	public String 		getMenuPath() 			{ return PanelPlugin.STATUS; }
	public int			getUpdateDelay() 		{ return 1000; }
	public int 			getNeededYacyRevision() { return 2920; }
	
	public void init(Peer peer, Config cfg) {
		this.peer = peer;
		this.cfg = cfg;
		title.setText(peer.getName());
		try {
			maxRamCache = peer.getRamCacheSize();
			maxMemory = peer.getMemoryMaxMB();
		} catch (RemoteException e) { cfg.log(e); }
	}
	
	public void doUpdate() {
		cfg.logDebug(" Updating Status.");
		
		long tLinks = 0;
		long tWords = 0;
		int tPPM = 0;
		int tpStatus = -1;
		String tType = null;
		int tpCountActive = 0;
		long tLinksActive = 0;
		long tWordsActive = 0;
		long memTotal = 0;
		long memFree = 0;
		long memMax = 0;
		int tRamCacheNow = 0;
		int tRamCacheSize = 0;
		
		tLinks = peer.getLinks();
		tWords = peer.getWords();
		tPPM = peer.getPPM();
		tpStatus = peer.getPEER_STATI();
		tType = peer.getType();
		tpCountActive = peer.getPeerCount(Peer.PEERS_ACTIVE);
		tLinksActive = peer.getLinks(Peer.PEERS_ACTIVE);
		tWordsActive = peer.getWords(Peer.PEERS_ACTIVE);
		memMax = peer.getMemoryMax();
		try {
			tRamCacheSize = peer.getRamCacheSize();
			tRamCacheNow = peer.getRamCacheNow();
			memFree = peer.getMemoryFree();
			memTotal = peer.getMemoryTotal();
		} catch (RemoteException e) { cfg.log(e); }
		
		links.setText(LongToSeparatedString(tLinks));
		words.setText(LongToSeparatedString(tWords));
		jPanel4.addValue(tPPM, 0);
		jPanel4.repaint();
		ppm.setText(Integer.toString(tPPM));
		pStatus.setText(pStati.get(tpStatus));
		pStatus.setForeground(pStatiColors.get(tpStatus));
		peerStatus.setText(pTypes.get(tType)[0]);
		peerStatus.setToolTipText(pTypes.get(tType)[1]);
		peerCount.setText(Integer.toString(tpCountActive));
		linkCount.setText(LongToSeparatedString(tLinksActive));
		wordCount.setText(LongToSeparatedString(tWordsActive));
		float memCFree = ((float)(memTotal - memFree) / (float)memMax);
		memory.setValue(Math.round(memCFree * 100));
		memory.setToolTipText("Freier Speicher: " + ((memMax - (memTotal - memFree)) / (1024 * 1024)) + " MB");
		aktMem.setForeground((memCFree > memWarn) ? WARNING : NORMAL);
		aktMem.setText(Long.toString((memTotal - memFree) / (1024  * 1024)) + " / " + Integer.toString(maxMemory) + " MB");
		ramCache.setValue(Math.round(((float)tRamCacheNow / (float)tRamCacheSize * 100)));
		aktRamCache.setText(LongToSeparatedString(tRamCacheNow) + " / " + LongToSeparatedString(maxRamCache) + " Words");
		
		jPanel.updateUI();
	}
	
	private String LongToSeparatedString(long l) {
		String oldnumber = Long.toString(l);
		if(oldnumber.length() < 4) return oldnumber;
		
		String s = "";		
		for(int i=oldnumber.length()-3;i>-3;i=i-3)
			s = oldnumber.substring((i>0)?i:0,i+3)+(s.length()>0?".":"")+s;
		
		return s;
	}

	private JPanel getJPanel() {
		if (jPanel == null) {
			GridBagConstraints gridBagConstraints23 = new GridBagConstraints();
			gridBagConstraints23.gridx = 1;
			gridBagConstraints23.anchor = GridBagConstraints.WEST;
			gridBagConstraints23.insets = new Insets(5, 5, 5, 5);
			gridBagConstraints23.gridy = 6;
			peerStatus = new JLabel("");
			GridBagConstraints gridBagConstraints17 = new GridBagConstraints();
			gridBagConstraints17.gridx = 0;
			gridBagConstraints17.insets = new Insets(5, 5, 5, 5);
			gridBagConstraints17.anchor = GridBagConstraints.EAST;
			gridBagConstraints17.gridy = 6;
			gridBagConstraints17.weightx = .5;
			jLabel1 = new JLabel("Peer Status:");
			GridBagConstraints gridBagConstraints22 = new GridBagConstraints();
			gridBagConstraints22.gridx = 2;
			gridBagConstraints22.insets = new Insets(5, 5, 5, 5);
			gridBagConstraints22.gridy = 5;
			gridBagConstraints22.weightx = .5;
			aktRamCache = new JLabel("");
			GridBagConstraints gridBagConstraints110 = new GridBagConstraints();
			gridBagConstraints110.gridx = 2;
			gridBagConstraints110.insets = new Insets(5, 5, 5, 5);
			gridBagConstraints110.gridy = 4;
			gridBagConstraints110.anchor = GridBagConstraints.WEST;
			gridBagConstraints110.weightx = .5;
			aktMem = new JLabel("");
			GridBagConstraints gridBagConstraints19 = new GridBagConstraints();
			gridBagConstraints19.gridx = 1;
			//gridBagConstraints19.fill = GridBagConstraints.BOTH;
			gridBagConstraints19.insets = new Insets(5, 5, 5, 5);
			gridBagConstraints19.gridy = 3;
			GridBagConstraints gridBagConstraints13 = new GridBagConstraints();
			gridBagConstraints13.gridx = 0;
			gridBagConstraints13.insets = new Insets(5, 5, 5, 5);
			gridBagConstraints13.gridwidth = 3;
			gridBagConstraints13.gridy = 0;
			gridBagConstraints13.weightx = .5;
			title = new JLabel("");
			title.setFont(new Font("Dialog", Font.BOLD, 18));
			GridBagConstraints gridBagConstraints15 = new GridBagConstraints();
			gridBagConstraints15.gridx = 1;
			gridBagConstraints15.insets = new Insets(5, 5, 5, 5);
			gridBagConstraints15.gridy = 5;
			gridBagConstraints15.weightx = 1;
			GridBagConstraints gridBagConstraints14 = new GridBagConstraints();
			gridBagConstraints14.gridx = 0;
			gridBagConstraints14.insets = new Insets(5, 5, 5, 5);
			gridBagConstraints14.anchor = GridBagConstraints.EAST;
			gridBagConstraints14.gridy = 5;
			gridBagConstraints14.weightx = .5;
			jLabel11 = new JLabel("RAM Cache:");
			GridBagConstraints gridBagConstraints11 = new GridBagConstraints();
			gridBagConstraints11.gridx = 2;
			gridBagConstraints11.insets = new Insets(5, 5, 5, 5);
			gridBagConstraints11.anchor = GridBagConstraints.WEST;
			gridBagConstraints11.gridy = 3;
			gridBagConstraints11.weightx = .5;
			ppm = new JLabel("");
			GridBagConstraints gridBagConstraints9 = new GridBagConstraints();
			gridBagConstraints9.gridx = 1;
			gridBagConstraints9.insets = new Insets(5, 5, 5, 5);
			gridBagConstraints9.anchor = GridBagConstraints.WEST;
			gridBagConstraints9.weighty = 0.0D;
			gridBagConstraints9.gridy = 7;
			pStatus = new JLabel("");
			GridBagConstraints gridBagConstraints8 = new GridBagConstraints();
			gridBagConstraints8.gridx = 0;
			gridBagConstraints8.insets = new Insets(5, 5, 5, 5);
			gridBagConstraints8.anchor = GridBagConstraints.EAST;
			gridBagConstraints8.gridy = 7;
			gridBagConstraints8.weightx = .5;
			jLabel6 = new JLabel("Verbindung:");
			GridBagConstraints gridBagConstraints7 = new GridBagConstraints();
			gridBagConstraints7.gridx = 1;
			gridBagConstraints7.insets = new Insets(5, 5, 5, 5);
			gridBagConstraints7.gridy = 4;
			gridBagConstraints8.weightx = 1;
			GridBagConstraints gridBagConstraints6 = new GridBagConstraints();
			gridBagConstraints6.gridx = 0;
			gridBagConstraints6.insets = new Insets(5, 5, 5, 5);
			gridBagConstraints6.anchor = GridBagConstraints.EAST;
			gridBagConstraints6.gridy = 4;
			gridBagConstraints6.weightx = .5;
			jLabel5 = new JLabel("Belegter Speicher:");
			GridBagConstraints gridBagConstraints4 = new GridBagConstraints();
			gridBagConstraints4.gridx = 0;
			gridBagConstraints4.insets = new Insets(5, 5, 5, 5);
			gridBagConstraints4.anchor = GridBagConstraints.EAST;
			gridBagConstraints4.gridy = 3;
			gridBagConstraints4.weightx = .5;
			jLabel4 = new JLabel("PPM:");
			jLabel4.setToolTipText("Pages per Minute");
			GridBagConstraints gridBagConstraints3 = new GridBagConstraints();
			gridBagConstraints3.gridx = 1;
			gridBagConstraints3.insets = new Insets(5, 5, 5, 5);
			gridBagConstraints3.anchor = GridBagConstraints.WEST;
			gridBagConstraints3.gridy = 2;
			words = new JLabel("");
			GridBagConstraints gridBagConstraints2 = new GridBagConstraints();
			gridBagConstraints2.gridx = 0;
			gridBagConstraints2.insets = new Insets(5, 5, 5, 5);
			gridBagConstraints2.anchor = GridBagConstraints.EAST;
			gridBagConstraints2.gridy = 2;
			gridBagConstraints2.weightx = .5;
			jLabel2 = new JLabel("Words:");
			GridBagConstraints gridBagConstraints1 = new GridBagConstraints();
			gridBagConstraints1.gridx = 1;
			gridBagConstraints1.insets = new Insets(5, 5, 5, 5);
			gridBagConstraints1.anchor = GridBagConstraints.WEST;
			gridBagConstraints1.gridy = 1;
			links = new JLabel("");
			GridBagConstraints gridBagConstraints = new GridBagConstraints();
			gridBagConstraints.gridx = 0;
			gridBagConstraints.insets = new Insets(5, 5, 5, 5);
			gridBagConstraints.anchor = GridBagConstraints.EAST;
			gridBagConstraints.gridy = 1;
			gridBagConstraints.weightx = .5;
			jLabel = new JLabel("Links:");
			jPanel = new JPanel();
			jPanel.setLayout(new GridBagLayout());
			// jPanel.setPreferredSize(new Dimension(0, 230));
			jPanel.add(jLabel, gridBagConstraints);
			jPanel.add(links, gridBagConstraints1);
			jPanel.add(jLabel2, gridBagConstraints2);
			jPanel.add(words, gridBagConstraints3);
			jPanel.add(jLabel4, gridBagConstraints4);
			jPanel.add(jLabel5, gridBagConstraints6);
			jPanel.add(getMemory(), gridBagConstraints7);
			jPanel.add(jLabel6, gridBagConstraints8);
			jPanel.add(pStatus, gridBagConstraints9);
			jPanel.add(ppm, gridBagConstraints11);
			jPanel.add(jLabel11, gridBagConstraints14);
			jPanel.add(getRamCache(), gridBagConstraints15);
			jPanel.add(title, gridBagConstraints13);
			jPanel.add(getJPanel4(), gridBagConstraints19);
			jPanel.add(aktMem, gridBagConstraints110);
			jPanel.add(aktRamCache, gridBagConstraints22);
			jPanel.add(jLabel1, gridBagConstraints17);
			jPanel.add(peerStatus, gridBagConstraints23);
		}
		return jPanel;
	}

	private JProgressBar getMemory() {
		if (memory == null) {
			memory = new JProgressBar();
			memory.setToolTipText("Maximal: ");
			memory.setMinimum(0);
			memory.setMaximum(100);
			memory.setMinimumSize(mid_dim);
			memory.setMaximumSize(mid_dim);
			memory.setPreferredSize(mid_dim);
		}
		return memory;
	}
	
	private JProgressBar getRamCache() {
		if (ramCache == null) {
			ramCache = new JProgressBar();
			ramCache.setPreferredSize(mid_dim);
			ramCache.setMaximumSize(mid_dim);
			ramCache.setMinimumSize(mid_dim);
		}
		return ramCache;
	}

	private JPanel getJPanel2() {
		if (jPanel2 == null) {
			wordCount = new JLabel("");
			linkCount = new JLabel("");
			peerCount = new JLabel("");
			jPanel2 = new JPanel();
			jPanel2.setLayout(new FlowLayout());
			jPanel2.add(new JLabel("Aktiv:"), null);
			jPanel2.add(peerCount, null);
			jPanel2.add(new JLabel("Peers"), null);
			jPanel2.add(linkCount, null);
			jPanel2.add(new JLabel("Links"), null);
			jPanel2.add(wordCount, null);
			jPanel2.add(new JLabel("Words"), null);
		}
		return jPanel2;
	}
	
	private JPopupMenu getPPMMenu() {
		if (ppmMenu == null) {
			ppmMenu = new JPopupMenu();
			JMenu x = new JMenu("Bereich");
			JMenuItem r;
			int[] field = new int[] { 1, 10, 30, 60, };
			for (int i=0; i<field.length; i++) {
				r = new JMenuItem(field[i] + " Sekunden");
				r.addActionListener(new alPPM(field[i]));
				x.add(r);
			}
			ppmMenu.add(x);
			JMenuItem y = new JMenuItem("Graph löschen");
			y.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					jPanel4.resetGraphs();
				}
			});
			ppmMenu.add(y);
		}
		return ppmMenu;
	}
	
	private class alPPM implements ActionListener {
		
		private final int seconds;
		
		public alPPM(int seconds) { this.seconds = seconds; }
		
		public void actionPerformed(ActionEvent e) {
			ArrayList<Double> ar = jPanel4.getValues(0);
			ArrayList<Double> r = new ArrayList<Double>();
			double d = 0;
			for (int i=0; i<ar.size(); i++) {
				if ((i + 1) % seconds != 0) {
					d += ar.get(i);
				} else {
					r.add(d / seconds);
					d = 0;
				}
			}
			jPanel4.putValues(0, r);
		}
	}
	
	private DGramm getJPanel4() {
		if (jPanel4 == null) {
			jPanel4 = new DGramm(1,50);
			jPanel4.setXFactor(1,0);
			jPanel4.setGraphFilled(true, 0);
			jPanel4.setPreferredSize(mid_dim1);
			jPanel4.setMaximumSize(mid_dim1);
			jPanel4.setMinimumSize(mid_dim1);
			jPanel4.setToolTipText("PPM gibt die Anzahl der indexierten Seiten pro Minute an.");
			jPanel4.addValue(15, 0);
			jPanel4.addValue(0, 0);
			jPanel4.addMouseListener(new MouseAdapter() {
				public void mouseClicked(MouseEvent e) {
					if (e.getButton() == MouseEvent.BUTTON3)	{	// rechte Maustaste
						getPPMMenu().show(jPanel4, e.getX(), e.getY());
					}
				}
			});
		}
		return jPanel4;
	}
	
}
