/* 
 * Autor: Franz Brauße
 * License: GPL v2 
 */

package tools;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.RenderingHints;
import java.util.ArrayList;
import java.util.Timer;
import javax.swing.JPanel;

public class DGramm extends JPanel {
	
	public static final long serialVersionUID = 1L;
	
	private static final Color[] graphColors = new Color[] {
		Color.LIGHT_GRAY, 	Color.GREEN, 	Color.BLUE,
		Color.PINK, 		Color.CYAN, 	Color.GRAY,
		Color.MAGENTA, 		Color.YELLOW, 	Color.WHITE };
	
	private Image img = null;
	private Graphics2D gr = null;
	
	private ArrayList<ArrayList<Double>> values = null;
	private ArrayList<Color> graphColor = null;
	private ArrayList<Color> graphBorder = null;
	private ArrayList<Boolean> graphFilled = null;
	
	private int maxValues = 0;
	private int gridX = 0;
	private int gridY = 0;
	private int loc = 0;
	
	private Timer tim = null;
	private Color gridColor = null;
	private Color bgColor = null;
	private Color textColor = null;
	private int textX = 0;
	private int textY = 0;
	private int graphNrForTextDisplay = 0;
	
	private boolean useGrid = false;
	private boolean showText = false;
	
	private String preText = null;
	private String postText = null;
	
	private int width = 0;
	private int height = 0;
	private boolean loaded = false;
	private ArrayList<Float> xfactor = null;		// stretch factor in x-direction
	
	/**
	 * DGramm... ne?!
	 * @param graphCount
	 * @param maxValues
	 */
	public DGramm(int graphCount, int maxValues) {
		super();
		super.repaint();
		if (maxValues < 1)
			throw new IllegalArgumentException("maxValues must not be smaller than 1");
		this.maxValues = maxValues;
		init(graphCount);
	}
	
	private void init(int graphCount) {
		this.values = new ArrayList<ArrayList<Double>>();
		this.graphColor = new ArrayList<Color>();
		this.graphBorder = new ArrayList<Color>();
		this.graphFilled = new ArrayList<Boolean>();
		this.xfactor = new ArrayList<Float>();
		for (int i=0; i<graphCount; i++) {
			this.values.add(new ArrayList<Double>());
			if (i<graphColors.length)
				this.graphColor.add(graphColors[i]);
			else
				this.graphColor.add(Color.DARK_GRAY);
			this.graphBorder.add(Color.GRAY);
			this.graphFilled.add(false);
			this.xfactor.add(1F);
		}
		this.maxValues = 50;
		this.gridColor = Color.GREEN;
		this.bgColor = Color.BLACK;
		this.textColor = Color.GRAY;
		this.preText = "";
		this.postText = "";
		this.textX = this.getWidth() - 50;
		this.textY = 5;
	}
	
	private boolean tryInit() {
		if (super.getWidth() == 0 || super.getHeight() == 0)
			return false;
		this.img = super.createImage(super.getWidth(),super.getHeight());
		if (this.img != null) {
			this.gr = (Graphics2D)this.img.getGraphics();
			this.width = super.getSize().width;
			this.height = super.getSize().height;
			this.gr.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
			if (this.maxValues == 0) this.maxValues = super.getSize().width;
			this.loaded = true;
		}
		return loaded;
	}
	
	public ArrayList<Double> getValues(int graphNr) {
		checkGraphCount(graphNr);
		return values.get(graphNr);
	}
	
	public void putValues(int graphNr, ArrayList<Double> values) {
		checkGraphCount(graphNr);
		this.values.remove(graphNr);
		this.values.add(graphNr, values);
		this.repaint();
	}
	
	private String getAktText(int graphNr) {
		checkGraphCount(graphNr);
		String r = Double.toString(this.values.get(graphNr).get(this.loc));
		if (this.preText.length() > 0)
			r = preText + r;
		if (this.postText.length() > 0)
			r += postText;
		return r;
	}
	
	private void paintGrid() {
		this.gr.setColor(this.gridColor);
		for (int i=this.loc % this.gridX; i<super.getWidth(); i+=this.gridX)
			this.gr.drawLine(i, 0, i, super.getHeight());
		for (int i=(int)getMax() % this.gridY; i<super.getHeight(); i+=this.gridY)
			this.gr.drawLine(0, i, super.getWidth(), i);
	}
	
	private double getMax() {
		checkGraphCount(0);
		double r = getMax(0);
		for (int i=1; i<this.values.size(); i++)
			r = Math.max(r, getMax(i));
		return r;
	}
	
	private double getMax(int graphNr) {
		checkGraphCount(graphNr);
		if (this.values.get(graphNr).size() == 0)
			return 0; //throw new ArrayIndexOutOfBoundsException("Graph 0 doesn't contain values");
		double r = this.values.get(graphNr).get(0);
		for (int i=1; i<this.values.get(graphNr).size(); i++)
			r = Math.max(r, this.values.get(graphNr).get(i));
		return r;
	}
	
	private double getMin() {
		checkGraphCount(0);
		double r = getMin(0);
		for (int i=1; i<this.values.size(); i++)
			r = Math.min(r, getMax(i));
		return r;
	}
	
	private double getMin(int graphNr) {
		checkGraphCount(graphNr);
		if (this.values.get(graphNr).size() == 0)
			return 0; //throw new ArrayIndexOutOfBoundsException("Graph 0 doesn't contain values");
		double r = this.values.get(graphNr).get(0);
		for (int i=1; i<this.values.get(graphNr).size(); i++)
			r = Math.min(r, this.values.get(graphNr).get(i));
		return r;
	}
	
	private void checkGraphCount(int graphNr) throws DGrammNoSuchGraphException {
		if (graphNr >= this.values.size())
			throw new DGrammNoSuchGraphException(graphNr);
	}
	
	/**
	 * 
	 * @param values
	 */
	public void addValues(double[] values) {
		if (values.length != this.values.size())
			throw new ArrayIndexOutOfBoundsException("values-count doesn't match graph count");
		for (int i=0; i<this.values.size(); i++)
			this.values.get(i).add(values[i]);
	}
	
	/**
	 * 
	 * @param value
	 * @param graphNr
	 */
	// TODO
	public void addValue(double value, int graphNr) {
		checkGraphCount(graphNr);
		double[] d = new double[this.values.size()];
		for (int i=0; i<d.length; i++)
			d[i] = 0;
		d[graphNr] = value;
		addValues(d);
	}
	
	/**
	 * 
	 * @param graphNr
	 */
	public void setGraphForTextDisplay(int graphNr) {
		checkGraphCount(graphNr);
		this.graphNrForTextDisplay = graphNr;
	}
	
	/**
	 * 
	 * @param textColor
	 */
	public void setTextColor(Color textColor) {
		this.textColor = textColor;
	}
	
	/**
	 * 
	 */
	public void resetGraphs() {
		for (int i=0; i<this.values.size(); i++)
			this.values.set(i, new ArrayList<Double>());
	}
	
	/**
	 * 
	 * @param graphNr
	 */
	public void resetGraph(int graphNr) {
		checkGraphCount(graphNr);
		this.values.set(graphNr, new ArrayList<Double>());
	}
	
	/**
	 * 
	 * @param value
	 * @param graphNr
	 */
	public void setXFactor(float value, int graphNr) {
		if (value <= 0)
			throw new IllegalArgumentException("value has to be greater than 0");
		checkGraphCount(graphNr);
		this.xfactor.set(graphNr, value);
	}
	
	/**
	 * 
	 * @param graphCount
	 */
	public void setGraphCount(int graphCount) {
		if (graphCount < 0)
			throw new IllegalArgumentException("graphCount must be greater or equal zero");
		
		if (graphCount >= this.values.size())
			for (int i=this.values.size(); i<graphCount; i++)
				addGraph(null, null, false, 1);
		else if (graphCount < this.values.size()-1)
			for (int i=this.values.size()-1; i>=graphCount; i--)
				removeGraph(i);
	}
	
	/**
	 * 
	 * @param borderColor
	 * @param filledColor
	 * @param filled
	 */
	public void addGraph(Color borderColor, Color filledColor, boolean filled, float XFactor) {
		this.values.add(new ArrayList<Double>());
		this.graphBorder.add(borderColor);
		this.graphColor.add(filledColor);
		this.graphFilled.add(filled);
		this.xfactor.add(XFactor);
	}
	
	/**
	 * 
	 * @param graphNr
	 */
	public void removeGraph(int graphNr) {
		checkGraphCount(graphNr);
		this.values.remove(graphNr);
		this.graphBorder.remove(graphNr);
		this.graphColor.remove(graphNr);
		this.graphFilled.remove(graphNr);
		this.xfactor.remove(graphNr);
	}
	
	/**
	 * 
	 * @param graphColors
	 */
	public void setGraphColors(Color[] graphColors) {
		if (graphColors.length != this.graphColor.size())
			throw new ArrayIndexOutOfBoundsException("values count doesn't match graph count");
		for (int i=0; i<this.graphColor.size(); i++)
			setGraphColor(i, graphColors[i]);
	}
	
	/**
	 * 
	 * @param yes
	 * @param graphNr
	 */
	// TODO
	public void setGraphFilled(boolean yes, int graphNr) {
		checkGraphCount(graphNr);
		this.graphFilled.set(graphNr, yes);
	}
	
	/**
	 * 
	 * @param graphNr
	 * @param graphColor
	 */
	public void setGraphColor(int graphNr, Color graphColor) {
		checkGraphCount(graphNr);
		this.graphColor.set(graphNr, graphColor);
	}
	
	/**
	 * 
	 * @param graphFilledColors
	 */
	public void setGraphBorderColors(Color[] graphFilledColors) {
		if (graphFilledColors.length != this.graphBorder.size())
			throw new ArrayIndexOutOfBoundsException("graphFilledColors count doesn't match graph count");
		for (int i=0; i<this.graphBorder.size(); i++)
			setGraphBorderColor(i, graphFilledColors[i]);
	}
	
	/**
	 * 
	 * @param graphNr
	 * @param graphFilledColor
	 */
	public void setGraphBorderColor(int graphNr, Color graphFilledColor) {
		checkGraphCount(graphNr);
		this.graphBorder.set(graphNr, graphFilledColor);
	}
	
	/**
	 * 
	 * @param timems
	 */
	public void setUpdateIntervall(int timems) {
		this.tim = new Timer();
		this.tim.schedule(new java.util.TimerTask() {
			public void run() {
				repaint();
			}
		}, 0, timems);
	}
	
	@Override
	public void paint(Graphics g) {
		if (tryInit()) {
			try {
				paintGraphs();
			} catch (ArithmeticException e) {
				/* ignore this */
			} catch (DGrammNoSuchGraphException e) {
				super.paint(g);
				return;
			}
			if (this.useGrid)
				paintGrid();
			if (this.showText)
				paintText();
			g.drawImage(this.img, 0, 0, this);
		} else {
			// g.drawImage(super.createImage(super.getWidth(), super.getHeight()), 0, 0, this);
			super.paint(g);
		}
	}
	
	private void paintGraphs() {
		double min = this.getMin();
		double max = this.getMax();
		this.gr.setColor(this.bgColor);
		this.gr.drawRect(0, 0, this.width, this.height);
		if (min == max)
			throw new ArithmeticException("Minimum value equals maximum value");
		double factor = super.getHeight() / (max - min);
		// System.out.println("Min: " + min + ", Max: " + max + ", Faktor: " + factor);
		int x, y, x0, y0, getStart;
		
		for (int i=0; i<this.values.size(); i++) {
			getStart = this.values.get(i).size() - (int)Math.floor(this.width / this.xfactor.get(i));
			if (getStart < 2)
				getStart = 1;	// prevent NPE, graph doesn't move
			else
				getStart--;		// draw one step over border because it looks more nicely
			for (int j=getStart; j<this.values.get(i).size(); j++) {
				x0 = Math.round((j - 1 - getStart) * this.xfactor.get(i));
				y0 = this.height - (int)Math.round((this.values.get(i).get(j - 1) - ((min >= 0) ? (min) : (max))) * factor);
				x = Math.round((j - getStart) * this.xfactor.get(i));
				y = this.height - (int)Math.round((this.values.get(i).get(j) - ((min >= 0) ? (min) : (max))) * factor);
				// System.out.print("i: " + i + ", j/X: " + x + ", Y org: " + this.values.get(i).get(j) + ", Y: " + y);
				
				if (this.graphFilled.get(i)) {
					this.gr.setColor(this.graphColor.get(i));
					int zero = (int)Math.round(((min > 0) ? (min) : ((max > 0) ? (max) : ((2 * max - min) ))) * factor);
					// System.out.print(", Zero: " + zero);
					// this.gr.drawLine(x, y + ((zero > y) ? (1) : (-1)), x, (int)Math.round(zero));
					this.gr.fillPolygon(
							new int[] { x0 + 1, x0 + 1, x + 1, x + 1 },
							new int[] {
									zero,
									y0 + ((zero > y0) ? (-1) : (1)),
									y + ((zero > y) ? (-1) : (1)),
									zero },
							4);
				}
				this.gr.setColor(this.graphBorder.get(i));
				this.gr.drawLine(x0, y0, x, y);
				// System.out.println();
			}
		}
	}
	
	private void paintText() {
		checkGraphCount(this.graphNrForTextDisplay);
		paintText(this.graphNrForTextDisplay);
	}
	
	private void paintText(int graphNr) {
		this.gr.setColor(this.textColor);
		this.gr.drawString(getAktText(graphNr), this.textX, this.textY);
	}
}
