/* 
 * Autor: Franz Brauße
 * License: GPL v2 
 */

package views;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;

import javax.swing.ButtonGroup;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JTabbedPane;
import javax.swing.JToggleButton;
import javax.swing.JComboBox;
import javax.swing.JSlider;
import javax.swing.JTextField;
import javax.swing.SwingConstants;
import javax.swing.event.CaretEvent;
import javax.swing.event.CaretListener;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import general.Config;

/**
 * This class represents the view for customizing the YacyAdmin settings.
 */
public class Settings extends JTabbedPane {
	
	private static final long serialVersionUID = 1L;
	
	private static final ImageIcon denied = new ImageIcon("ressources/remove.png");
	
	public static final String LBL_CONNECTIONS 				= "Verbindungen";
	public static final String LBL_UPDATE_INTERVAL_ 		= "Update Interval:";
	public static final String LBL_AUTO_CONNECTION_ 		= "Automatische Verbindung:";
	public static final String LBL_CONNECTION_ 				= "Verbindung:";
	public static final String LBL_CONECTION_TIMEOUT_ 		= "Verbindungs-Timeout:";
	public static final String LBL_LOG_SHOW_ 				= "Log anzeigen:";
	public static final String LBL_LOH_HIDE_ 				= "Log ausblenden:";
	public static final String LBL_DISPLAY_ 				= "Display:";
	public static final String LBL_HOST_ 					= "Host:";
	public static final String LBL_PORT_ 					= "Port:";
	public static final String LBL_SECONDS_ABBREV 			= "sec";
	public static final String LBL_APPLY 					= "\u00DCbernehmen";
	public static final String LBL_NO_CONNECTION 			= "Keine Verbindung";
	public static final String LBL_OFF1 					= "aus";
	public static final String LBL_UNLIMITED 				= "unendlich";
	public static final String LBL_ALWAYS_ON_TOP 			= "Immer im Vordergrund";
	public static final String LBL_HTTP 					= "http";
	public static final String LBL_HTTPS 					= "https";
	public static final String LBL_ALWAYS 					= "immer";
	public static final String LBL_USE_HTTP11 				= "HTTP 1.1 verwenden";
	public static final String LBL_USE_CHUNKED_TRANSFERS 	= "Chunked Transfers verwenden";
	public static final String LBL_USE_GZIP_COMPRESSION 	= "GZip Kompression verwenden";
	public static final String LBL_USE_PROXY 				= "Proxy benutzen";
	public static final String LBL_INVALID_PORT 			= "Port-Angabe ist ung\u00FCltig";
	
	private static final Insets STD_INSETS = new Insets(5, 5, 5, 5);
	
	private ButtonGroup 	logShow 			= new ButtonGroup();
	private JButton 		btnSubmit 			= null;
	private JCheckBox 		jCHttp11 			= null;
	private JCheckBox 		jCChunkedTransfers 	= null;
	private JCheckBox 		jCGzip 				= null;
	private JCheckBox 		jCBUpdateInterval 	= null;
	private JCheckBox 		jCProxy 			= null;
	private JCheckBox 		jCBAlwaysOnTop 		= null;
	private JComboBox 		jCAutoConnect 		= null;
	private JComboBox 		jCProtcol 			= null;
	private JLabel 			jLUpdInterval 		= null;
	private JLabel 			jLHideLogDelay 		= null;
	private JLabel 			jLConnectionTimeout = null;
	private JLabel 			jLPortStatus 		= null;
	private JRadioButton 	jRLogShowAlway 		= null;
	private JRadioButton 	jRLogHide 			= null;
	private JSlider 		jSLogHideDelay 		= null;
	private JSlider 		jSUpdateInterval 	= null;
	private JSlider 		jSConnectionTimeout = null;
	private JTextField 		jTHost 				= null;
	private JTextField 		jTPort 				= null;
	
	private Config 			cfg 				= null;

	public Settings(Config cfg) {
		super();
		this.cfg = cfg;
		initialize();
	}
	
	private void initialize() {
		
		// X: 1
		GridBagConstraints gridBagConstraints6 = new GridBagConstraints();
		gridBagConstraints6.fill = GridBagConstraints.HORIZONTAL;
		gridBagConstraints6.gridx = 1;
		gridBagConstraints6.gridy = 1;
		gridBagConstraints6.weightx = 1.0;
		gridBagConstraints6.gridwidth = 4;
		gridBagConstraints6.insets = STD_INSETS;
		GridBagConstraints gridBagConstraints14 = new GridBagConstraints();
		gridBagConstraints14.fill = GridBagConstraints.HORIZONTAL;
		gridBagConstraints14.gridy = 9;
		gridBagConstraints14.weightx = 1.0;
		gridBagConstraints14.gridwidth = 3;
		gridBagConstraints14.insets = STD_INSETS;
		gridBagConstraints14.gridx = 1;
		
		// X: 2
		GridBagConstraints gridBagConstraints41 = new GridBagConstraints();
		gridBagConstraints41.fill = GridBagConstraints.HORIZONTAL;
		gridBagConstraints41.gridx = 2;
		gridBagConstraints41.gridy = 0;
		gridBagConstraints41.weightx = 1.0;
		gridBagConstraints41.gridwidth = 2;
		gridBagConstraints41.insets = STD_INSETS;
		GridBagConstraints gridBagConstraints52 = new GridBagConstraints();
		gridBagConstraints52.fill = GridBagConstraints.HORIZONTAL;
		gridBagConstraints52.gridy = 11;
		gridBagConstraints52.weightx = 1.0;
		gridBagConstraints52.gridwidth = 2;
		gridBagConstraints52.insets = STD_INSETS;
		gridBagConstraints52.gridx = 2;
		
		// X: 3
		GridBagConstraints gridBagConstraints53 = new GridBagConstraints();
		gridBagConstraints53.fill = GridBagConstraints.HORIZONTAL;
		gridBagConstraints53.gridy = 8;
		gridBagConstraints53.weightx = 1.0;
		gridBagConstraints53.insets = STD_INSETS;
		gridBagConstraints53.gridwidth = 1;
		gridBagConstraints53.gridx = 3;
		GridBagConstraints gridBagConstraints43 = new GridBagConstraints();
		gridBagConstraints43.fill = GridBagConstraints.HORIZONTAL;
		gridBagConstraints43.gridy = 7;
		gridBagConstraints43.weightx = 1.0;
		gridBagConstraints43.insets = STD_INSETS;
		gridBagConstraints43.gridwidth = 2;
		gridBagConstraints43.gridx = 3;
		
		// X: 4
		GridBagConstraints gridBagConstraints62 = new GridBagConstraints();
		gridBagConstraints62.gridx = 4;
		gridBagConstraints62.gridy = 8;
		GridBagConstraints gridBagConstraints33 = new GridBagConstraints();
		gridBagConstraints33.gridx = 4;
		gridBagConstraints33.fill = GridBagConstraints.HORIZONTAL;
		gridBagConstraints33.insets = STD_INSETS;
		gridBagConstraints33.gridy = 9;
		GridBagConstraints gridBagConstraints61 = new GridBagConstraints();
		gridBagConstraints61.gridx = 4;
		gridBagConstraints61.insets = STD_INSETS;
		gridBagConstraints61.fill = GridBagConstraints.HORIZONTAL;
		gridBagConstraints61.gridy = 11;
		
		GridBagConstraints gridBagConstraints7 = new GridBagConstraints();
		gridBagConstraints7.fill = GridBagConstraints.HORIZONTAL;
		gridBagConstraints7.gridx = 1;
		gridBagConstraints7.gridy = 2;
		gridBagConstraints7.weightx = 1.0;
		gridBagConstraints7.gridwidth = 4;
		gridBagConstraints7.anchor = GridBagConstraints.WEST;
		gridBagConstraints7.insets = STD_INSETS;
		
		GridBagConstraints gridBagConstraints24 = new GridBagConstraints();
		gridBagConstraints24.gridx = 1;
		gridBagConstraints24.insets = STD_INSETS;
		gridBagConstraints24.anchor = GridBagConstraints.WEST;
		gridBagConstraints24.gridwidth = 2;
		gridBagConstraints24.gridy = 7;
		GridBagConstraints gridBagConstraints34 = new GridBagConstraints();
		gridBagConstraints34.gridx = 1;
		gridBagConstraints34.insets = STD_INSETS;
		gridBagConstraints34.gridwidth = 2;
		gridBagConstraints34.anchor = GridBagConstraints.WEST;
		gridBagConstraints34.gridy = 8;
		
		GridBagConstraints gridBagConstraints21 = new GridBagConstraints();
		gridBagConstraints21.gridx = 1;
		gridBagConstraints21.gridy = 0;
		GridBagConstraints gridBagConstraints42 = new GridBagConstraints();
		gridBagConstraints42.gridx = 1;
		gridBagConstraints42.gridy = 11;
		
		GridBagConstraints gridBagConstraints12 = new GridBagConstraints();
		gridBagConstraints12.gridx = 1;
		gridBagConstraints12.gridwidth = 4;
		gridBagConstraints12.anchor = GridBagConstraints.WEST;
		gridBagConstraints12.gridy = 3;
		GridBagConstraints gridBagConstraints31 = new GridBagConstraints();
		gridBagConstraints31.gridx = 1;
		gridBagConstraints31.gridwidth = 3;
		gridBagConstraints31.anchor = GridBagConstraints.WEST;
		gridBagConstraints31.gridy = 10;
		
		GridBagConstraints gridBagConstraints8 = new GridBagConstraints();
		gridBagConstraints8.gridx = 4;
		gridBagConstraints8.insets = STD_INSETS;
		gridBagConstraints8.gridy = 0;
		
		GridBagConstraints gbcRLabels = new GridBagConstraints();
		gbcRLabels.gridx = 0;
		gbcRLabels.insets = STD_INSETS;
		gbcRLabels.anchor = GridBagConstraints.EAST;
		gbcRLabels.gridy = 0;
		
		jLUpdInterval = new JLabel("3.0 " + LBL_SECONDS_ABBREV);
		jLUpdInterval.setHorizontalAlignment(SwingConstants.LEFT);
		jLUpdInterval.setMinimumSize(new Dimension(65, 15));
		jLUpdInterval.setMaximumSize(new Dimension(65, 15));
		jLUpdInterval.setPreferredSize(new Dimension(65, 15));
		jLPortStatus = new JLabel("");
		jLPortStatus.setPreferredSize(new Dimension(16, 16));
		jLHideLogDelay = new JLabel("");
		jLConnectionTimeout = new JLabel(Float.toString(((float)cfg.timeout / 1000F)) + " " + LBL_SECONDS_ABBREV);
		
		JPanel connPanel = new JPanel();
		connPanel.setLayout(new GridBagLayout());
		connPanel.add(getJSUpdateInterval(), gridBagConstraints41);
		connPanel.add(getJCAutoConnect(), gridBagConstraints6);
		connPanel.add(getJCProtcol(), gridBagConstraints7);
		connPanel.add(jLUpdInterval, gridBagConstraints8);
		connPanel.add(getJCBUpdateInterval(), gridBagConstraints21);
		connPanel.add(getJRLogShowAlway(), gridBagConstraints31);
		connPanel.add(getJRLogHide(), gridBagConstraints42);
		connPanel.add(getJSLogHideDelay(), gridBagConstraints52);
		connPanel.add(jLHideLogDelay, gridBagConstraints61);
		connPanel.add(getJSConnectionTimeout(), gridBagConstraints14);
		connPanel.add(jLConnectionTimeout, gridBagConstraints33);
		connPanel.add(new JLabel(LBL_HOST_), gridBagConstraints24);
		connPanel.add(new JLabel(LBL_PORT_), gridBagConstraints34);
		connPanel.add(getJTHost(), gridBagConstraints43);
		connPanel.add(getJTPort(), gridBagConstraints53);
		connPanel.add(jLPortStatus, gridBagConstraints62);
		
		addJComponents(new JComponent[] { new JLabel(LBL_UPDATE_INTERVAL_), new JLabel(LBL_AUTO_CONNECTION_), new JLabel(LBL_CONNECTION_) }, gbcRLabels, true, connPanel);
		gbcRLabels.gridy = 9;
		addJComponents(new JComponent[] { new JLabel(LBL_CONECTION_TIMEOUT_), new JLabel(LBL_LOG_SHOW_), new JLabel(LBL_LOH_HIDE_), new JLabel(LBL_DISPLAY_) }, gbcRLabels, true, connPanel);
		addJComponents(new JComponent[] { getJCHttp11(), getJCChunkedTransfers(), getJCGzip(), getJCProxy() }, gridBagConstraints12, true, connPanel);

		gbcRLabels.gridx = 3;
		gbcRLabels.gridy = 13;
		gbcRLabels.gridwidth = 2;
		gbcRLabels.weightx = 0;
		gbcRLabels.anchor = GridBagConstraints.EAST;
		connPanel.add(getBtnSubmit(), gbcRLabels);
		
		gbcRLabels.gridx = 1;
		gbcRLabels.gridy = 12;
		gbcRLabels.gridwidth = 4;
		gbcRLabels.anchor = GridBagConstraints.WEST;
		gbcRLabels.insets = new Insets(0, 0, 0, 0);
		connPanel.add(getJCBAlwaysOnTop(), gbcRLabels);
		
		this.addTab(LBL_CONNECTIONS, connPanel);
		this.setSize(449, 427);
		
		updateProxy();
	}
	
	private JButton getBtnSubmit() {
		if (btnSubmit == null) {
			btnSubmit = new JButton(LBL_APPLY);
			btnSubmit.setEnabled(false);
			btnSubmit.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					cfg.setAlwaysOnTop(Settings.this.jCBAlwaysOnTop.isSelected());
					cfg.connectOnStartup = (jCAutoConnect.getSelectedItem().toString().equals("Keine Verbindung"))
							? ""
							: jCAutoConnect.getSelectedItem().toString();
					cfg.useHTTPS = jCProtcol.getSelectedItem().toString().equals("https");
					cfg.globalUpdateDelay = jSUpdateInterval.getValue() * 500;
					cfg.useGlobalUpdateDelay = jCBUpdateInterval.isSelected();
					cfg.globalUpdateDelay = jSUpdateInterval.getValue() * 500;
					cfg.logShowAlways = jRLogShowAlway.isSelected();
					cfg.log.show();
					cfg.logShowAlways = !jRLogHide.isSelected();
					if (jRLogHide.isSelected())
						cfg.log.closeAfterDelay();
					cfg.logHideDelay = jSLogHideDelay.getValue() * 500;
					cfg.useHTTP11 = jCHttp11.isSelected();
					cfg.timeout = jSConnectionTimeout.getValue() * 100;
					cfg.useChunkedTransfer = jCChunkedTransfers.isSelected();
					cfg.useGZipEncoding = jCGzip.isSelected();
					Settings.this.btnSubmit.setEnabled(false);
				}
			});
		}
		return btnSubmit;
	}
	
	private static void addJComponents(JComponent[] jc,
			GridBagConstraints gbc, boolean vertical, JPanel to) {
		
		if (jc == null || gbc == null || to == null) return;
		for (int i=0; i<jc.length; i++) {
			to.add(jc[i], gbc);
			if (vertical) gbc.gridy++; else gbc.gridx++;
		}
	}
	
	private class settingsListener implements ActionListener, ChangeListener,
			ItemListener, CaretListener {
		
		public void actionPerformed(ActionEvent e) {
			Settings.this.btnSubmit.setEnabled(true);
			jSUpdateInterval.setEnabled(jCBUpdateInterval.isSelected());
			jLUpdInterval.setEnabled(jCBUpdateInterval.isSelected());
			updateProxy();
		}
		
		public void stateChanged(ChangeEvent e) {
			Settings.this.btnSubmit.setEnabled(true);
			jLUpdInterval.setText((jSUpdateInterval.getValue() == 0)
					? (LBL_OFF1)
					: Float.toString((float)jSUpdateInterval.getValue() / 2F) + " " + LBL_SECONDS_ABBREV);
			jLHideLogDelay.setText(Float.toString((float)jSLogHideDelay.getValue() / 2F) + " " + LBL_SECONDS_ABBREV);
			jLConnectionTimeout.setText((jSConnectionTimeout.getValue() == 0)
					? (LBL_UNLIMITED)
					: ((float)jSConnectionTimeout.getValue() / 10F) + " " + LBL_SECONDS_ABBREV);
		}
		
		public void itemStateChanged(ItemEvent e) {
			Settings.this.btnSubmit.setEnabled(true);
			jSLogHideDelay.setEnabled(!jRLogShowAlway.isSelected());
			jLHideLogDelay.setEnabled(!jRLogShowAlway.isSelected());
		}
		
		public void caretUpdate(CaretEvent e) {
			Settings.this.btnSubmit.setEnabled(true);
			updateProxy();
		}
	}
	
	private JCheckBox getJCBAlwaysOnTop() {
		if (jCBAlwaysOnTop == null) {
			jCBAlwaysOnTop = new JCheckBox(LBL_ALWAYS_ON_TOP);
			jCBAlwaysOnTop.setSelected(cfg.isAlwaysOnTop());
			jCBAlwaysOnTop.addActionListener(new settingsListener());
		}
		return jCBAlwaysOnTop;
	}
	
	private JComboBox getJCAutoConnect() {
		if (jCAutoConnect == null) {
			jCAutoConnect = new JComboBox();
			jCAutoConnect.setPreferredSize(new Dimension(32, 19));
			jCAutoConnect.setMaximumSize(new Dimension(32767, 19));
			jCAutoConnect.setMinimumSize(new Dimension(32, 19));
			jCAutoConnect.addItem(LBL_NO_CONNECTION);
			int x = 0;
			boolean found = false; 
			for (String d : cfg.connects.keySet()) {
				jCAutoConnect.addItem(d);
				if (d.equals(cfg.connectOnStartup)) found = true;
				if (!found) x++;
			}
			jCAutoConnect.setSelectedIndex((found) ? (++x) : (0));
			jCAutoConnect.addActionListener(new settingsListener());
		}
		return jCAutoConnect;
	}

	private JComboBox getJCProtcol() {
		if (jCProtcol == null) {
			jCProtcol = new JComboBox();
			jCProtcol.setPreferredSize(new Dimension(32, 19));
			jCProtcol.setMaximumSize(new Dimension(32767, 19));
			jCProtcol.setMinimumSize(new Dimension(32, 19));
			jCProtcol.addItem(LBL_HTTP);
			jCProtcol.addItem(LBL_HTTPS);
			jCProtcol.setSelectedIndex((cfg.useHTTPS) ? (1) : (0));
			jCProtcol.addActionListener(new settingsListener());
		}
		return jCProtcol;
	}
	
	private JSlider getJSUpdateInterval() {
		if (jSUpdateInterval == null) {
			jSUpdateInterval = new JSlider();
			jSUpdateInterval.setMaximum(20);
			jSUpdateInterval.setValue(cfg.globalUpdateDelay / 500);
			jLUpdInterval.setText(Float.toString((float)jSUpdateInterval.getValue() / 2F) + " " + LBL_SECONDS_ABBREV);
			jSUpdateInterval.addChangeListener(new settingsListener());
		}
		return jSUpdateInterval;
	}

	private JCheckBox getJCBUpdateInterval() {
		if (jCBUpdateInterval == null) {
			jCBUpdateInterval = new JCheckBox();
			jCBUpdateInterval.setSelected(cfg.useGlobalUpdateDelay);
			jSUpdateInterval.setEnabled(jCBUpdateInterval.isSelected());
			jLUpdInterval.setEnabled(jCBUpdateInterval.isSelected());
			jCBUpdateInterval.addActionListener(new settingsListener());
		}
		return jCBUpdateInterval;
	}

	private JRadioButton getJRLogShowAlway() {
		if (jRLogShowAlway == null) {
			JToggleButton.ToggleButtonModel toggleButtonModel = new JToggleButton.ToggleButtonModel();
			toggleButtonModel.setGroup(logShow);
			jRLogShowAlway = new JRadioButton();
			jRLogShowAlway.setText(LBL_ALWAYS);
			jRLogShowAlway.setModel(toggleButtonModel);
			jRLogShowAlway.setSelected(cfg.logShowAlways);
			jRLogShowAlway.addItemListener(new settingsListener());
		}
		return jRLogShowAlway;
	}

	private JRadioButton getJRLogHide() {
		if (jRLogHide == null) {
			JToggleButton.ToggleButtonModel toggleButtonModel1 = new JToggleButton.ToggleButtonModel();
			toggleButtonModel1.setGroup(logShow);
			jRLogHide = new JRadioButton();
			jRLogHide.setModel(toggleButtonModel1);
			jRLogHide.setSelected(!cfg.logShowAlways);
			jRLogHide.addItemListener(new settingsListener());
		}
		return jRLogHide;
	}

	private JSlider getJSLogHideDelay() {
		if (jSLogHideDelay == null) {
			jSLogHideDelay = new JSlider();
			jSLogHideDelay.setValue(6);
			jSLogHideDelay.setMinimum(1);
			jSLogHideDelay.setMaximum(20);
			jSLogHideDelay.setValue(cfg.logHideDelay / 500);
			jLHideLogDelay.setText(Float.toString((float)cfg.logHideDelay / 1000F) + " " + LBL_SECONDS_ABBREV);
			jSLogHideDelay.setEnabled(!jRLogShowAlway.isSelected());
			jLHideLogDelay.setEnabled(!jRLogShowAlway.isSelected());
			jSLogHideDelay.addChangeListener(new settingsListener());
		}
		return jSLogHideDelay;
	}

	private JCheckBox getJCHttp11() {
		if (jCHttp11 == null) {
			jCHttp11 = new JCheckBox();
			jCHttp11.setText(LBL_USE_HTTP11);
			jCHttp11.setSelected(cfg.useHTTP11);
			jCHttp11.addActionListener(new settingsListener());
		}
		return jCHttp11;
	}

	private JCheckBox getJCChunkedTransfers() {
		if (jCChunkedTransfers == null) {
			jCChunkedTransfers = new JCheckBox();
			jCChunkedTransfers.setText(LBL_USE_CHUNKED_TRANSFERS);
			jCChunkedTransfers.setSelected(cfg.useChunkedTransfer);
			jCChunkedTransfers.addActionListener(new settingsListener());
		}
		return jCChunkedTransfers;
	}
	
	private JCheckBox getJCGzip() {
		if (jCGzip == null) {
			jCGzip = new JCheckBox();
			jCGzip.setText(LBL_USE_GZIP_COMPRESSION);
			jCGzip.setSelected(cfg.useGZipEncoding);
			jCGzip.addActionListener(new settingsListener());
		}
		return jCGzip;
	}

	private JSlider getJSConnectionTimeout() {
		if (jSConnectionTimeout == null) {
			jSConnectionTimeout = new JSlider();
			jSConnectionTimeout.setValue(100);
			jSConnectionTimeout.setMaximum(600);
			jSConnectionTimeout.setValue(Math.round(cfg.timeout / 100F));
			jSConnectionTimeout.addChangeListener(new settingsListener());
		}
		return jSConnectionTimeout;
	}
	
	private JCheckBox getJCProxy() {
		if (jCProxy == null) {
			jCProxy = new JCheckBox();
			jCProxy.setSelected(cfg.isProxyEnabled());
			jCProxy.setText(LBL_USE_PROXY);
			jCProxy.addActionListener(new settingsListener());
			
		}
		return jCProxy;
	}
	
	private void updateProxy() {
		jTHost.setEnabled(jCProxy.isSelected());
		jTPort.setEnabled(jCProxy.isSelected());
		boolean ok = (
				jTPort.getText().matches("\\d+") &&
				Integer.valueOf(jTPort.getText()) > 0 &&
				jTHost.getText().length() > 0);
		cfg.setProxy((ok) ? (jTHost.getText()) : (null), (ok) ? (Integer.valueOf(jTPort.getText())) : (0));
		cfg.setProxyEnabled(jCProxy.isSelected());
		jLPortStatus.setIcon((ok || !jCProxy.isSelected()) ? (null) : (denied));
		jLPortStatus.setToolTipText((ok || !jCProxy.isSelected()) ? ("") : (LBL_INVALID_PORT));
	}
	
	private JTextField getJTHost() {
		if (jTHost == null) {
			jTHost = new JTextField();
			jTHost.setText(cfg.getProxyHost());
			jTHost.addCaretListener(new settingsListener());
		}
		return jTHost;
	}
	
	private JTextField getJTPort() {
		if (jTPort == null) {
			jTPort = new JTextField();
			jTPort.setText(Integer.toString(cfg.getProxyPort()));
			jTPort.addCaretListener(new settingsListener());
		}
		return jTPort;
	}
	
}
