/* 
 * Autor: Franz Brauße
 * License: GPL v2 
 */

package yacy;

import java.security.MessageDigest;

/* from yacy */

public final class Codings {
	
    private static final char[] alpha = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/".toCharArray();
    
    public static byte[] encodeMD5Raw(String key) {
    	try {
    		MessageDigest digest = MessageDigest.getInstance("MD5");
    		digest.reset();
    		digest.update(key.getBytes());
    		return digest.digest();
    	} catch (java.security.NoSuchAlgorithmException e) {
    		System.out.println("Internal Error at md5:" + e.getMessage());
    	}
    	return null;
    }
    
    public static String encodeMD5Hex(String key) {
    	// generate a hex representation from the md5 of a string
    	return encodeHex(encodeMD5Raw(key));
    }
    
    public static String encodeHex(byte[] in) {
    	if (in == null) return "";
    	String result = "";
    	for (int i = 0; i < in.length; i++)
    		result = result + (((0Xff & (int) in[i]) < 16) ? "0" : "") + Integer.toHexString(0Xff & (int) in[i]);
    	return result;
    }
    
	public static final String encodeString(String in) {
        return encode(in.getBytes());
    }

    // we will use this encoding to encode strings with 2^8 values to
    // b64-Strings
    // we will do that by grouping each three input bytes to four output bytes.
    private static final String encode(byte[] in) {
        StringBuffer out = new StringBuffer(in.length / 3 * 4 + 3);
        int pos = 0;
        long l;
        while (in.length - pos >= 3) {
            l = ((((0XffL & (long) in[pos]) << 8) + (0XffL & (long) in[pos + 1])) << 8) + (0XffL & (long) in[pos + 2]);
            pos += 3;
            out = out.append(encodeLong(l, 4));
        }
        // now there may be remaining bytes
        if (in.length % 3 != 0)
            out = out.append((in.length % 3 == 2) ? encodeLong((((0XffL & (long) in[pos]) << 8) + (0XffL & (long) in[pos + 1])) << 8, 4).substring(0, 3) : encodeLong((((0XffL & (long) in[pos])) << 8) << 8, 4).substring(0, 2));
        while (out.length() % 4 > 0)
            out.append("=");
        // return result
        return out.toString();
    }
	
    public static final String encodeLong(long c, int length) {
        StringBuffer s = new StringBuffer(length);
        s.setLength(length);
        while (length > 0) {
            s.setCharAt(--length, alpha[(byte) (c & 0x3F)]);
            c >>= 6;
        }
        return s.toString();
    }
}
