/* 
 * Autor: Franz Brauße
 * License: GPL v2 
 */

package yacy.datatypes;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.jdom.Element;

public final class TransferAction {
	
	/* Regexes taken from YaCy Logalizer, svn://svn.berlios.de/yacy/yacylogalizer */
	/* slightly modified */
	private static final String yacyHash = "[\\w-_]{12}";
	private static final String valueI = "\\d+";
	private static final String valueF = "[\\d.-]*";
	
	// RegExp for LogLevel I
    private static final Pattern rEntries = Pattern.compile(
    		"Received ("+valueI+") Entries ("+valueI+") Words \\["+yacyHash+" .. "+yacyHash+"\\]/"+valueF+" from ("+yacyHash+"):" +
    		"[\\w-_]*/"+valueF+", processed in ("+valueI+") milliseconds, requesting ("+valueI+")/("+valueI+") URLs, blocked " +
    		"("+valueI+") RWIs");
    private static final Pattern rUrls = Pattern.compile(
    		"Received ("+valueI+") URLs from peer ("+yacyHash+"):[\\w-_]*/"+valueF+" in ("+valueI+") ms, Blocked ("+valueI+") "+
    		"URLs");
    private static final Pattern tWords = Pattern.compile(
    		"Index transfer of ("+valueI+") entries ("+valueI+") words \\["+yacyHash+" .. "+yacyHash+"\\] and ("+valueI+") URLs "+
    		"to peer ([\\w-_]*):("+yacyHash+") in ("+valueI+") seconds successful \\(("+valueI+") words/s, ("+valueI+") Bytes\\)");
    private static final Pattern tEnd = Pattern.compile("DHT distribution: transfer to peer ([\\w-]*) finished.");
    
	private static final HashMap<String,Matcher> openReceives = new HashMap<String,Matcher>();
	private static final HashMap<String,Matcher> openTransfers = new HashMap<String,Matcher>();
	
	public static void clearOpenReceives() { openReceives.clear(); }
	public static void clearOpenTransfers() { openTransfers.clear(); }
	
	public static ArrayList<TransferAction> genTransferActions(ArrayList<Element> logEntries) {
		if (logEntries == null) return null;
		ArrayList<TransferAction> r = new ArrayList<TransferAction>();
		String msg;
		Matcher m1, m2;
		for (Element d : logEntries) {
			msg = d.getChildText("message");

			m1 = rEntries.matcher(msg);
			if (m1.matches()) {
				openReceives.put(m1.group(3), m1);
			}
			
			m2 =rUrls.matcher(msg);
			if (m2.matches()) {
				if ((m1 = openReceives.get(m2.group(2))) == null) continue;
				r.add(new TransferAction(
						true,
						Integer.valueOf(m1.group(1)),
						Integer.valueOf(m1.group(2)),
						Integer.valueOf(m2.group(1)),
						Integer.valueOf(m1.group(4)) + Integer.valueOf(m2.group(3)),
						m2.group(2),
						-1,
						new Date(Long.valueOf(d.getChildText("millis")))));
				openReceives.remove(m2.group(2));
			}
			
			m1 = tWords.matcher(msg);
			if (m1.matches()) {
				openTransfers.put(m1.group(4), m1);
			}
			
			m2 = tEnd.matcher(msg);
			if (m2.matches()) {
				if ((m1 = openTransfers.get(m2.group(1))) == null) continue;
				r.add(new TransferAction(
						false,
						Integer.valueOf(m1.group(1)),
						Integer.valueOf(m1.group(2)),
						Integer.valueOf(m1.group(3)),
						Long.valueOf(m1.group(6)) * 1000,
						m1.group(5),
						Integer.valueOf(m1.group(8)),
						new Date(Long.valueOf(d.getChildText("millis")))));
				openTransfers.remove(m1.group(4));
			}
		}
		return r;
	}
	
	/* ------------------------------------------------------------------------------- */
	
	private boolean incoming;
	private int 	ec;
	private int 	wc;
	private int 	uc;
	private long 	duration;
	private String 	remoteHash;
	private int 	bytes;
	private Date 	date;
	
	public TransferAction(
			boolean incoming,
			int entryCount,
			int wordCount,
			int urlCount,
			long duration,
			String remoteHash,
			int bytes,
			Date date) {
		this.incoming = incoming;
		this.ec = entryCount;
		this.wc = wordCount;
		this.uc = urlCount;
		this.duration = duration;
		this.remoteHash = remoteHash;
		this.bytes = bytes;
		this.date = date;
	}
	
	public boolean 	incoming() 		{ return this.incoming; }
	public int 		getEntryCount() { return this.ec; }
	public int 		getWordCount() 	{ return this.wc; }
	public int 		getUrlCount() 	{ return this.uc; }
	public long		getDuration() 	{ return this.duration; }
	public String 	getRemoteHash() { return this.remoteHash; }
	public int 		getBytes() 		{ return this.bytes; }
	public Date 	getDate() 		{ return this.date; }
}
