/* 
 * Autor: Franz Brauße
 * License: GPL v2 
 */

package yacy.event;

import java.util.EventListener;

import yacy.exceptions.PInterruptionEx;


public interface PeerEventListener extends EventListener {
	
	/**
	 * <p>This event is thrown when a setting or value has changed.</p>
	 * 
	 * @see PeerEvent
	 * @param event the event thrown by the peer
	 */
	public void peerEvent(PeerEvent event);
	
	/**
	 * <p>Thrown when the peer is shut down or the connection is going to be terminated.
	 * Cancel this event via it's method <code>cancelEvent</code> to request time to
	 * finish your current transaction with the peer.</p>
	 * <p>The triggering method will receive a {@link yacy.exceptions.PInterruptionEx}
	 * and may then decide whether to override your request or not.</p>
	 * 
	 * @see PeerInterruptableEvent#interrupt(String, Thread)
	 * @see PInterruptionEx#overrideInterruption()
	 * @param event the interruptable event thrown by the peer
	 */
	public void peerInterruptableEvent(PeerInterruptableEvent event);
}
